import sys
import optparse

from ase.cli.run import Runner, str2dict

from gpaw import GPAW
from gpaw.mixer import Mixer, MixerSum
from gpaw.occupations import FermiDirac, MethfesselPaxton
from gpaw.wavefunctions.pw import PW


class GPAWRunner(Runner):
    def __init__(self):
        Runner.__init__(self)
        self.calculator_name = 'gpaw'

    def make_parser(self):
        parser = optparse.OptionParser(
            usage='gwap run [options] [system, ...]',
            description='Run calculation for system(s).')
        return parser
        
    def add_options(self, parser):
        Runner.add_options(self, parser)
        parser.add_option('-w', '--write', help='Write gpw-file.')
        parser.add_option('-W', '--write-all',
                          help='Write gpw-file with wave functions.')

    def set_calculator(self, atoms, name):
        parameter_namespace = {
            'PW': PW,
            'FermiDirac': FermiDirac,
            'MethfesselPaxton': MethfesselPaxton,
            'Mixer': Mixer,
            'MixerSum': MixerSum}
        parameters = str2dict(self.opts.parameters, parameter_namespace)
        atoms.calc = GPAW(txt=self.get_filename(name, 'txt'), **parameters)

    def calculate(self, atoms, name):
        data = Runner.calculate(self, atoms, name)
        if self.opts.write:
            atoms.calc.write(self.opts.write)
        if self.opts.write_all:
            atoms.calc.write(self.opts.write_all, 'all')
        return data
        
        
def main(args=None):
    runner = GPAWRunner()
    runner.parse(args)
    if runner.errors:
        sys.exit(runner.errors)



import numpy as np
from ase.units import Hartree
from gpaw.transport.tools import (aa1d, interpolate_array,
                                  distribute_atomic_matrices)
from gpaw.transport.io import Transport_IO
from gpaw.grid_descriptor import GridDescriptor
from gpaw import parsize_domain
from gpaw.domain import decompose_domain
'''
     ---------------------------------------
      side    |                     |  side
          o  o| o  o  o  o  o  o  o |o  o
       -      |                     |     +
          o  o| o  o  o  o  o  o  o |o  o
              |                     |
       ---------------------------------------
         Left |                     |Right
         Lead | <-- Scattering -->  |Lead
              |       Region        |

 class Surrounding is used to deal with the projection close to the
 boundary and combine the potential or density information of the
 scattering region and leads.
'''


class Side:
    """Describe the electrode boundary"""
    def __init__(self, type, direction, kpt_comm, domain_comm, h):
        # Direction: '-' for the left electrode, '+' for the right electrode

        self.type = type
        self.direction = direction
        self.kpt_comm = kpt_comm
        self.domain_comm = domain_comm
        self.tio = Transport_IO(kpt_comm, domain_comm)
        self.h_cz = h

    def abstract_boundary(self):
        # Abtract the effective potential, hartree potential, and average
        # density out from the electrode calculation.
        map = {'-': '0', '+': '1'}
        data = self.tio.read_data(filename='Lead_' +
                                  map[self.direction], option='Lead')
        nn = data['fine_N_c'][2]
        N_c = data['N_c']
        cell_cv = data['cell_cv']
        pbc_c = data['pbc_c']
        if isinstance(parsize_domain, int):
            parsize_c = None
            assert parsize_domain == self.domain_comm.size
        else:
            parsize_c = parsize_domain
        if parsize_c is None:
            parsize_c = decompose_domain(N_c, self.domain_comm.size)
        parsize_c = np.array(parsize_c)
       
        vHt_g = data['vHt_g']
        vt_sg = data['vt_sg']
        nt_sg = data['nt_sg']
        rhot_g = data['rhot_g']
        vt_sG = data['vt_sG']
        nt_sG = data['nt_sG']
        self.D_asp = data['D_asp']
        self.dH_asp = data['dH_asp']
        gd = GridDescriptor(N_c, cell_cv, pbc_c, self.domain_comm, parsize_c)
        finegd = gd.refine()

        self.boundary_vHt_g = None
        self.boundary_vHt_g1 = None
        self.boundary_vt_sg_line = None
        self.boundary_nt_sg = None
        self.boundary_rhot_g_line = None
        self.boundary_vt_sG = None
        self.boundary_nt_sG = None

        if self.tio.domain_comm.rank == 0:
            self.boundary_vHt_g = self.slice(nn, vHt_g)
            self.boundary_nt_sg = self.slice(nn, nt_sg)
            if self.direction == '-':
                other_direction = '+'
            else:
                other_direction = '-'
            h = self.h_cz / 2.0
            b_vHt_g0 = self.boundary_vHt_g.copy()
            b_vHt_g1 = self.boundary_vHt_g.copy()

            self.boundary_vHt_g = interpolate_array(b_vHt_g0,
                                                    finegd, h,
                                                    self.direction)
            self.boundary_vHt_g1 = interpolate_array(b_vHt_g1,
                                                     finegd, h,
                                                     other_direction)
            vt_sg = interpolate_array(vt_sg, finegd,
                                      h, self.direction)
            self.boundary_vt_sg_line = aa1d(vt_sg)
            self.boundary_nt_sg = interpolate_array(self.boundary_nt_sg,
                                                    finegd, h, self.direction)
            rhot_g = interpolate_array(rhot_g, finegd, h, self.direction)
            self.boundary_rhot_g_line = aa1d(rhot_g)

            nn //= 2
            h *= 2
            self.boundary_vt_sG = self.slice(nn, vt_sG)
            self.boundary_nt_sG = self.slice(nn, nt_sG)
            self.boundary_vt_sG = interpolate_array(self.boundary_vt_sG,
                                                    gd, h, self.direction)
            self.boundary_nt_sG = interpolate_array(self.boundary_nt_sG,
                                                    gd, h, self.direction)

    def slice(self, nn, in_array):
        if self.type == 'LR':
            seq1 = np.arange(nn)
            seq2 = np.arange(nn)
            di = len(in_array.shape) - 1
            if self.direction == '-':
                slice_array = np.take(in_array, seq1, axis=di)
            else:
                slice_array = np.take(in_array, seq2, axis=di)
        return slice_array


class Surrounding:
    """The potential and density environment of the scattering region"""
    def __init__(self, tp, type='LR'):
        self.type = type
        self.lead_num = tp.lead_num
        self.initialize(tp)

    def initialize(self, tp):
        if self.type == 'LR':
            self.sides = {}
            self.bias_index = {}
            self.side_basis_index = {}
            self.directions = ['-', '+']
            for i in range(self.lead_num):
                direction = self.directions[i]
                kpt_comm = tp.wfs.kd.comm
                gd_comm = tp.gd.comm
                side = Side('LR', direction, kpt_comm,
                            gd_comm, tp.gd.h_cv[2, 2])
                self.sides[direction] = side
                self.bias_index[direction] = tp.bias[i]
                self.side_basis_index[direction] = tp.lead_index[i]
            self.nn = tp.bnc[:]
            self.nn = np.array(self.nn)
            self.operator = tp.extended_calc.hamiltonian.poisson.operators[0]
        elif self.type == 'all':
            raise NotImplementedError
        self.calculate_sides()
        self.initialized = True

    def reset_bias(self, tp):
        self.bias = tp.bias
        for i in range(self.lead_num):
            direction = self.directions[i]
            self.bias_index[direction] = self.bias[i]
        self.combine(tp)

    def calculate_sides(self):
        if self.type == 'LR':
            for name, in self.sides:
                self.sides[name].abstract_boundary()
        if self.type == 'all':
            raise NotImplementedError('type all not yet')

    def get_extra_density(self, tp, vHt_g):
        """help to solve poisson euqation for different left and right
        electrode"""
        if self.type == 'LR':
            rhot_g = tp.finegd1.zeros()
            self.operator.apply(vHt_g, rhot_g)
            nn = self.nn * 2
            self.extra_rhot_g = self.uncapsule(tp, nn, rhot_g, tp.finegd1,
                                               tp.finegd)

    def capsule(self, tp, nn, loc_in_array, in_cap_array, gd, gd0):
        ns = tp.nspins
        cap_array = gd.collect(in_cap_array)
        in_array = gd0.collect(loc_in_array)

        if gd.comm.rank == 0:
            if len(loc_in_array.shape) == 4:
                local_cap_array = gd.zeros(ns)
                cap_array[:, :, :, nn[0]:-nn[1]] = in_array
            else:
                local_cap_array = gd.zeros()
                cap_array[:, :, nn[0]:-nn[1]] = in_array
        else:
            if len(loc_in_array.shape) == 4:
                local_cap_array = gd.zeros(ns)
            else:
                local_cap_array = gd.zeros()
        gd.distribute(cap_array, local_cap_array)
        return local_cap_array

    def uncapsule(self, tp, nn, in_array, gd, gd0):
        nn1 = nn[0]
        nn2 = nn[1]
        ns = tp.nspins
        di = 2
        if len(in_array.shape) == 4:
            di += 1
            local_uncap_array = gd0.zeros(ns)
        else:
            local_uncap_array = gd0.zeros()
        global_in_array = gd.collect(in_array)
        if gd.comm.rank == 0:
            seq = np.arange(nn1, global_in_array.shape[di] - nn2)
            uncap_array = np.take(global_in_array, seq, axis=di)
        else:
            uncap_array = None
        gd0.distribute(uncap_array, local_uncap_array)
        return local_uncap_array

    def combine(self, tp):
        if self.type == 'LR':
            nn = self.nn * 2
            ham = tp.extended_calc.hamiltonian
            if ham.vt_sg is None:
                ham.vt_sg = ham.finegd.empty(ham.nspins)
                ham.vHt_g = ham.finegd.zeros()
                ham.vt_sG = ham.gd.zeros(ham.nspins)
                ham.poisson.initialize()
                if not tp.fixed:
                    tp.inner_poisson.initialize()

            bias_shift0 = self.bias_index['-'] / Hartree
            bias_shift1 = self.bias_index['+'] / Hartree
            b_vHt_g0 = self.sides['-'].boundary_vHt_g
            b_vHt_g1 = self.sides['+'].boundary_vHt_g
            b_vHt_g01 = self.sides['-'].boundary_vHt_g1
            b_vHt_g11 = self.sides['+'].boundary_vHt_g1
            if tp.fixed:
                if tp.gd.comm.rank == 0:
                    tp.inner_poisson.initialize(b_vHt_g0 + bias_shift0,
                                                b_vHt_g1 + bias_shift1)
                else:
                    tp.inner_poisson.initialize(None, None)

            if tp.gd.comm.rank == 0:
                vHt_g = ham.finegd.zeros(global_array=True)
                extra_vHt_g = ham.finegd.zeros(global_array=True)
                nt_sg = ham.finegd.zeros(tp.nspins, global_array=True)
                nt_sG = ham.gd.zeros(tp.nspins, global_array=True)

                vHt_g[:, :, :nn[0]] = b_vHt_g0 + bias_shift0
                vHt_g[:, :, -nn[1]:] = b_vHt_g1 + bias_shift1

                mnn = np.min(nn)
                extra_vHt_g[:, :, nn[0] - mnn:nn[0]] = (
                    bias_shift0 + b_vHt_g0[:, :, -mnn:] -
                    b_vHt_g11[:, :, -mnn:])
                if nn[1] != mnn:
                    extra_vHt_g[:, :, -nn[1]:-nn[1] + mnn] = (
                        bias_shift1 +
                        b_vHt_g1[:, :, :mnn] - b_vHt_g01[:, :, :mnn])
                else:
                    extra_vHt_g[:, :, -nn[1]:] = (
                        bias_shift1 +
                        b_vHt_g1[:, :, :mnn] - b_vHt_g01[:, :, :mnn])

                nt_sg[:, :, :, :nn[0]] = self.sides['-'].boundary_nt_sg
                nt_sg[:, :, :, -nn[1]:] = self.sides['+'].boundary_nt_sg

                nn //= 2
                nt_sG[:, :, :, :nn[0]] = self.sides['-'].boundary_nt_sG
                nt_sG[:, :, :, -nn[1]:] = self.sides['+'].boundary_nt_sG
            else:
                nt_sG = None
                nt_sg = None
                vHt_g = None
                extra_vHt_g = None

            loc_extra_vHt_g = ham.finegd.zeros()
            self.nt_sg = ham.finegd.zeros(tp.nspins)
            self.nt_sG = ham.gd.zeros(tp.nspins)

            ham.gd.distribute(nt_sG, self.nt_sG)
            ham.finegd.distribute(nt_sg, self.nt_sg)
            ham.finegd.distribute(vHt_g, ham.vHt_g)
            ham.finegd.distribute(extra_vHt_g, loc_extra_vHt_g)
            self.get_extra_density(tp, loc_extra_vHt_g)

    def combine_vHt_g(self, tp, vHt_g):
        nn = self.nn * 2
        extended_vHt_g = tp.extended_calc.hamiltonian.vHt_g
        tp.extended_calc.hamiltonian.vHt_g = self.capsule(tp, nn, vHt_g,
                                                          extended_vHt_g,
                                                          tp.finegd1,
                                                          tp.finegd)

    def combine_nt_sG(self, tp, nt_sG):
        nn = self.nn
        self.nt_sG = self.capsule(tp, nn, nt_sG, self.nt_sG, tp.gd1,
                                  tp.gd)
        return self.nt_sG

    def combine_dH_asp(self, tp, dH_asp):
        ham = tp.extended_calc.hamiltonian
        all_dH_asp = dH_asp[:]
        for i in range(self.lead_num):
            direction = self.directions[i]
            side = self.sides[direction]
            for dH_ap in side.dH_asp:
                all_dH_asp.append(dH_ap)
        ham.dH_asp = {}
        for a, D_sp in tp.extended_D_asp.items():
            ham.dH_asp[a] = np.zeros_like(D_sp)
        distribute_atomic_matrices(all_dH_asp, ham.dH_asp, ham.setups)

    def refresh_vt_sG(self, tp):
        nn = self.nn
        gd = tp.extended_calc.gd
        bias_shift0 = self.bias_index['-'] / Hartree
        bias_shift1 = self.bias_index['+'] / Hartree
        vt_sG = gd.collect(tp.extended_calc.hamiltonian.vt_sG)
        if gd.comm.rank == 0:
            vt_sG[:, :, :, :nn[0]] = (self.sides['-'].boundary_vt_sG +
                                      bias_shift0)
            vt_sG[:, :, :, -nn[1]:] = (self.sides['+'].boundary_vt_sG +
                                       bias_shift1)
        gd.distribute(vt_sG, tp.extended_calc.hamiltonian.vt_sG)



import numpy as np
from ase.io.pupynere import NetCDFFile


class Reader:
    def __init__(self, filename, comm):
        self.nc = NetCDFFile(filename)

    def dimension(self, name):
        return self.nc.dimensions[name]
    
    def __getitem__(self, name):
        value = getattr(self.nc, name)
        if isinstance(value, str):
            try:
                value = eval(value)
            except (SyntaxError, NameError):
                pass
            return value
        else:
            return value[0]

    def has_array(self, name):
        return name in self.nc.variables
    
    def get(self, name, *indices):
        var = self.nc.variables[name]
        if var.shape == ():
            return var.getValue()
        else:
            if var.dimensions[-1] == 'two':
                x = var[indices]
                array = np.empty(x.shape[:-1], complex)
                array.real = x[..., 0]
                array.imag = x[..., 1]
                return array
            else:
                return var[indices]

    def get_reference(self, name, indices):
        return NetCDFReference(self.nc.variables[name], indices)
    
    def close(self):
        self.nc.close()


class NetCDFReference:
    def __init__(self, var, indices):
        self.var = var
        self.indices = indices
        self.cmplx = (var.dimensions[-1] == 'two')
        n = len(indices)
        if self.cmplx:
            self.shape = var.shape[n:-1]
        else:
            self.shape = var.shape[n:]

    def __len__(self):
        return self.shape[0]

    def __getitem__(self, indices):
        if not isinstance(indices, tuple):
            indices = (indices,)
        if self.cmplx:
            x = self.var[self.indices + indices]
            array = np.empty(x.shape[:-1], complex)
            array.real = x[..., 0]
            array.imag = x[..., 1]
            return array
        else:
            return self.var[self.indices + indices]



import os
import time
import pickle as pckl

import numpy as np

from ase import Atoms
from ase.io import read
from ase.lattice.compounds import Rocksalt

from gpaw import GPAW, FermiDirac, Mixer
from gpaw.utilities import compiled_with_sl
from gpaw.eigensolvers import Davidson
from gpaw.wavefunctions.pw import PW
from gpaw.occupations import FermiDirac
from gpaw.response.df import DielectricFunction
from gpaw.mpi import world

# This test assures that some things that
# should be equal, are.

a = 2.5
c = 3.22

GR = Atoms(symbols='C2',
           positions=[(0.5*a, 0.2 + -np.sqrt(3) / 6 * a, 0.0),
                      (0.5*a, 0.2 + np.sqrt(3) / 6 * a, 0.0)],
           cell=[(0.5*a,-0.5*3**0.5*a,0),
                 (0.5*a,+0.5*3**0.5*a,0),
                 (0.0,0.0,c*2.0)])
GR.set_pbc((True,True,True))
atoms = GR
GSsettings = [{'symmetry': 'off', 'kpts': {'density': 2.5, 'gamma': False}},
              {'symmetry': {}, 'kpts': {'density': 2.5, 'gamma': False}},
              {'symmetry': 'off', 'kpts': {'density': 2.5, 'gamma': True}},
              {'symmetry': {}, 'kpts': {'density': 2.5, 'gamma': True}}]

DFsettings = [{'disable_point_group': True,
               'disable_time_reversal': True,
               'use_more_memory': 0},
              {'disable_point_group': False,
               'disable_time_reversal': True,
               'use_more_memory': 0},
              {'disable_point_group': True,
               'disable_time_reversal': False,
               'use_more_memory': 0},
              {'disable_point_group': False,
               'disable_time_reversal': False,
               'use_more_memory': 0},
              {'disable_point_group': False,
               'disable_time_reversal': False,
               'use_more_memory': 0,
               'unsymmetrized': False},
              {'disable_point_group': False,
               'disable_time_reversal': False,
               'use_more_memory': 1}]

if world.size > 1 and compiled_with_sl():
    DFsettings.append({'disable_point_group': False,
                       'disable_time_reversal': False,
                       'use_more_memory': 1,
                       'nblocks': 2})
              
for GSkwargs in GSsettings:
    calc = GPAW(h=0.18,
                mode=PW(600),
                occupations=FermiDirac(0.2),
                **GSkwargs)
 
    atoms.set_calculator(calc)
    atoms.get_potential_energy()
    calc.write('gr.gpw', 'all')

    dfs = []
    for kwargs in DFsettings:
        DF = DielectricFunction(calc='gr.gpw',
                                domega0=0.2,
                                eta=0.2,
                                ecut=40.0,
                                **kwargs)
        df1, df2 = DF.get_dielectric_function()
        if world.rank == 0:
            dfs.append(df1)

    while len(dfs):
        df = dfs.pop()
        for df2 in dfs:
            try:
                assert np.allclose(df, df2)
            except AssertionError:
                print(np.max(np.abs((df - df2) / df)))
                raise AssertionError




from __future__ import print_function
"""Test automatically write out of restart files"""

import os
import sys
import time
from gpaw import GPAW
from ase import Atom, Atoms
from gpaw.test import equal
from ase.parallel import rank, barrier, size

endings = ['gpw']
try:
    import Scientific.IO.NetCDFXXXX
    endings.append('nc')
except ImportError:
    pass

for ending in endings:
    restart = 'gpaw-restart-wfs_io.' + ending
    restart_wf = 'gpaw-restart-wfs_io-wf.' + ending
    # H2
    H = Atoms([Atom('H', (0, 0, 0)), Atom('H', (0, 0, 1))])
    H.center(vacuum=2.0)

    wfdir = 'wfs_tmp'
    mode = ending+':' + wfdir + '/psit_s%dk%dn%d'

    if 1:
        calc = GPAW(nbands=2, convergence={'eigenstates': 1e-3})
        H.set_calculator(calc)
        H.get_potential_energy()
        barrier()
        calc.write(restart_wf, 'all')
        calc.write(restart, mode)
        barrier()
        
    # refine the restart file containing the wfs 
    E1 = GPAW(restart_wf,
              convergence=
              {'eigenstates': 1.e-5}).get_atoms().get_potential_energy()
        
    # refine the restart file and separate wfs 
    calc = GPAW(restart, convergence={'eigenstates': 1.e-5})
    calc.read_wave_functions(mode)
    E2 = calc.get_atoms().get_potential_energy()

    print(E1, E2)
    equal(E1, E2, 1e-12)

    barrier()
    
    if rank == 0:
        os.remove(restart_wf)
        os.remove(restart)
        for f in os.listdir(wfdir):
            os.remove(wfdir + '/' + f)
        os.rmdir(wfdir)



from __future__ import print_function
from ase.lattice import bulk
from sys import argv
from ase.dft.kpoints import ibz_points, get_bandpath
from gpaw.eigensolvers.davidson import Davidson
from gpaw import *
from ase import *
from gpaw.test import gen
from gpaw import setup_paths
from gpaw.mpi import world

import os

"""This calculation has the following structure.
1) Calculate the ground state of Diamond.
2) Calculate the band structure of diamond in order to obtain accurate KS band gap for Diamond.
3) Calculate ground state again, and calculate the potential discontinuity using accurate band gap.
4) Calculate band structure again, and apply the discontinuity to CBM.

Compare to reference.
"""

xc = 'GLLBSC'
gen('C',xcname=xc)
setup_paths.insert(0, '.')

# Calculate ground state
atoms = bulk('C', 'diamond', a=3.567)
# We want sufficiently many grid points that the calculator
# can use wfs.world for the finegd, to test that part of the code.
calc = GPAW(h=0.13, kpts=(4,4,4), xc=xc, nbands=8,
            parallel=dict(domain=min(world.size, 2),
                          band=1),
            eigensolver=Davidson(niter=4))
atoms.set_calculator(calc)
atoms.get_potential_energy()
calc.write('Cgs.gpw')

# Calculate accurate KS-band gap from band structure
points = ibz_points['fcc']

# CMB is in G-X
G = points['Gamma']
X = points['X']

#W = points['W']
#K = points['K']
#L = points['L']
#[W, L, G, X, W, K]

kpts, x, X = get_bandpath([G, X], atoms.cell, npoints=12)
calc = GPAW('Cgs.gpw', kpts=kpts, fixdensity=True, symmetry='off',
            nbands=8,
            convergence=dict(bands=6), eigensolver=Davidson(niter=4))
calc.get_atoms().get_potential_energy()
# Get the accurate KS-band gap
homolumo = calc.occupations.get_homo_lumo(calc.wfs)
homo, lumo = homolumo
print("band gap ",(lumo-homo)*27.2)
    
# Redo the ground state calculation
calc = GPAW(h=0.15, kpts=(4,4,4), xc=xc, nbands=8,
            eigensolver=Davidson(niter=4))
atoms.set_calculator(calc)
atoms.get_potential_energy()
# And calculate the discontinuity potential with accurate band gap
response = calc.hamiltonian.xc.xcs['RESPONSE']
response.calculate_delta_xc(homolumo=homolumo)
calc.write('CGLLBSC.gpw')

# Redo the band structure calculation
atoms, calc = restart('CGLLBSC.gpw', kpts=kpts, fixdensity=True, symmetry='off',
                      convergence=dict(bands=6), eigensolver=Davidson(niter=2))
atoms.get_potential_energy()
response = calc.hamiltonian.xc.xcs['RESPONSE']
KS, dxc = response.calculate_delta_xc_perturbation()

assert abs(KS+dxc-5.41)<0.10
#M. Kuisma et. al, Phys. Rev. B 82, 115106, QP gap for C, 5.41eV, expt. 5.48eV



from __future__ import print_function
from gpaw import GPAW
from ase.structure import molecule
from gpaw.mpi import world

system = molecule('H2O')
system.center(vacuum=1.2)
system.pbc = 1

band = 1
if world.size > 4:
    band = 2

for mode in ['fd',
             #'pw',
             'lcao'
             ]:
    energy = []
    if mode != 'lcao' and band == 2:
        eigensolver = 'rmm-diis'
    else:
        eigensolver = None

    for augment_grids in [True, False]:
        calc = GPAW(mode=mode,
                    eigensolver=eigensolver,
                    parallel=dict(augment_grids=augment_grids,
                                  band=band),
                    basis='szp(dzp)',
                    kpts=[1, 1, 4],
                    nbands=8)
        def stopcalc():
            calc.scf.converged = True
        # Iterate enough for density to update so it depends on potential
        calc.attach(stopcalc, 3 if mode == 'lcao' else 5)
        system.set_calculator(calc)
        energy.append(system.get_potential_energy())
    err = energy[1] - energy[0]
    assert err < 1e-10, err



import os
from gpaw import GPAW, restart
from ase import Atoms
from gpaw.test import equal
from gpaw.mpi import world, rank
from math import sqrt
import numpy as np

# Test the reading of wave functions as file references

modes = ['gpw']
try:
    import _gpaw_hdf5
    modes.append('hdf5')
except ImportError:
    pass

d = 3.0
atoms = Atoms('Na3', positions=[( 0, 0, 0),
                              ( 0, 0, d),
                              ( 0, d*sqrt(3./4.), d/2.)],
                   magmoms=[1.0, 1.0, 1.0],
                   cell=(3.5, 3.5, 4.+2/3.),
                   pbc=True)

# Only a short, non-converged calcuation
conv = {'eigenstates': 1.24, 'energy':2e-1, 'density':1e-1}
calc = GPAW(h=0.30, kpts=(1,1,3), 
            setups={'Na': '1'},
            nbands=3, convergence=conv)
atoms.set_calculator(calc)
e0 = atoms.get_potential_energy()
wf0 = calc.get_pseudo_wave_function(2, 1, 1, broadcast=True)
# Write the restart file(s)
for mode in modes:
    calc.write('tmp.%s' % mode, 'all')

del calc
# Now read with single process
comm = world.new_communicator(np.array((rank,)))
for mode in modes:
    calc = GPAW('tmp.%s' % mode, communicator=comm)
    wf1 = calc.get_pseudo_wave_function(2, 1, 1)
    diff = np.abs(wf0 - wf1)
    assert(np.all(diff < 1e-12))

world.barrier()
if rank == 0:
    for mode in modes:
        os.remove('tmp.%s' % mode)



import numpy as np
from scipy.optimize import leastsq
import pylab as pl

def check_degenerate_bands(filename, etol):

    from gpaw import GPAW
    calc = GPAW(filename,txt=None)
    print('Number of Electrons   :', calc.get_number_of_electrons())
    nibzkpt = calc.get_ibz_k_points().shape[0]
    nbands = calc.get_number_of_bands()
    print('Number of Bands       :', nbands)
    print('Number of ibz-kpoints :', nibzkpt)
    e_kn = np.array([calc.get_eigenvalues(k) for k in range(nibzkpt)])
    f_kn = np.array([calc.get_occupation_numbers(k) for k in range(nibzkpt)])
    for k in range(nibzkpt):
        for n in range(1,nbands):
            if (f_kn[k,n-1] - f_kn[k,n] > 1e-5) and (np.abs(e_kn[k,n] - e_kn[k,n-1]) < etol):
                print(k, n, e_kn[k,n], e_kn[k, n-1])
    return
    

def get_orbitals(calc):
    """Get LCAO orbitals on 3D grid by lcao_to_grid method."""

    bfs_a = [setup.phit_j for setup in calc.wfs.setups]
    
    from gpaw.lfc import BasisFunctions
    bfs = BasisFunctions(calc.wfs.gd, bfs_a, calc.wfs.kd.comm, cut=True)
    spos_ac = calc.atoms.get_scaled_positions()
    bfs.set_positions(spos_ac)

    nLCAO = calc.get_number_of_bands()
    orb_MG = calc.wfs.gd.zeros(nLCAO)
    C_M = np.identity(nLCAO)
    bfs.lcao_to_grid(C_M, orb_MG,q=-1)
    
    return orb_MG

def find_peaks(x,y,threshold = None):
    """ Find peaks for a certain curve.

    Usage:
    threshold = (xmin, xmax, ymin, ymax)

    """

    assert isinstance(x, np.ndarray) and isinstance(y, np.ndarray)
    assert x.ndim == 1 and y.ndim == 1
    assert x.shape[0] == y.shape[0]

    if threshold is None:
        threshold = (x.min(), x.max(), y.min(), y.max())
        
    if not isinstance(threshold, tuple):
        threshold = (threshold, )

    if len(threshold) == 1:
        threshold += (x.max(), y.min(), y.max())
    elif len(threshold) == 2:
        threshold += (y.min(), y.max())
    elif len(threshold) == 3:
        threshold += (y.max(),)
    else:
        pass

    xmin = threshold[0]
    xmax = threshold[1]
    ymin = threshold[2]
    ymax = threshold[3]

    peak = {}
    npeak = 0
    for i in range(1, x.shape[0]-1):
        if (y[i] >= ymin and y[i] <= ymax and
            x[i] >= xmin and x[i] <= xmax ):
            if y[i] > y[i-1] and y[i] > y[i+1]:
                peak[npeak] = np.array([x[i], y[i]])
                npeak += 1

    peakarray = np.zeros([npeak, 2])
    for i in range(npeak):
        peakarray[i] = peak[i]
        
    return peakarray

def lorz_fit(x,y, npeak=1, initpara = None):
    """ Fit curve using Lorentzian function

    Note: currently only valid for one and two lorentizian

    The lorentzian function is defined as::

                      A w
        lorz = --------------------- + y0
                (x-x0)**2 + w**2

    where A is the peak amplitude, w is the width, (x0,y0) the peak position

    Parameters:

    x, y: ndarray
        Input data for analyze
    p: ndarray
        Parameters for curving fitting function. [A, x0, y0, w]
    p0: ndarray
        Parameters for initial guessing. similar to p
    
    """

    
    def residual(p, x, y):
        
        err = y - lorz(x, p, npeak)
        return err

    def lorz(x, p, npeak):

        if npeak == 1:        
            return p[0] * p[3] / ( (x-p[1])**2 + p[3]**2 ) + p[2]
        if npeak == 2:
            return ( p[0] * p[3] / ( (x-p[1])**2 + p[3]**2 ) + p[2]
                   + p[4] * p[7] / ( (x-p[5])**2 + p[7]**2 ) + p[6] )
        else:
            raise ValueError('Larger than 2 peaks not supported yet!')

    if initpara is None:
        if npeak == 1:
            initpara[i] = np.array([1., 0., 0., 0.1])
        if npeak == 2:
            initpara[i] = np.array([1., 0., 0., 0.1,
                                    3., 0., 0., 0.1])
    p0 = initpara
    
    result = leastsq(residual, p0, args=(x, y), maxfev=2000)

    yfit = lorz(x, result[0],npeak)

    return yfit, result[0]

def linear_fit(x,y, initpara = None):
    
    def residual(p, x, y):
        err = y - linear(x, p)
        return err

    def linear(x, p):
        return p[0]*x + p[1]

    if initpara is None:
        initpara = np.array([1.0,1.0])

    p0 = initpara
    result = leastsq(residual, p0, args=(x, y), maxfev=2000)
    yfit = linear(x, result[0])

    return yfit, result[0]


def plot_setfont():

    params = {
              'axes.labelsize': 18,
              'text.fontsize': 18,
              'legend.fontsize': 18,
              'xtick.labelsize': 18,
              'ytick.labelsize': 18,
              'text.usetex': True}
    #          'figure.figsize': fig_size}
    pl.rcParams.update(params)


def plot_setticks(x=True, y=True):

    pl.minorticks_on()
    ax = pl.gca()
    if x:
        ax.xaxis.set_major_locator(pl.AutoLocator())
        x_major = ax.xaxis.get_majorticklocs()
        dx_minor =  (x_major[-1]-x_major[0])/(len(x_major)-1) /5.
        ax.xaxis.set_minor_locator(pl.MultipleLocator(dx_minor))
    else:
        pl.minorticks_off()
        
    if y:
        ax.yaxis.set_major_locator(pl.AutoLocator())
        y_major = ax.yaxis.get_majorticklocs()
        dy_minor =  (y_major[-1]-y_major[0])/(len(y_major)-1) /5.
        ax.yaxis.set_minor_locator(pl.MultipleLocator(dy_minor)) 
    else:
        pl.minorticks_off()



from __future__ import print_function, division

import sys
from time import ctime

import numpy as np
from ase.units import Hartree
from ase.utils.timing import timer

import gpaw.mpi as mpi
from gpaw import extra_parameters
from gpaw.blacs import (BlacsGrid, BlacsDescriptor, Redistributor,
                        DryRunBlacsGrid)
from gpaw.kpt_descriptor import KPointDescriptor
from gpaw.occupations import FermiDirac
from gpaw.response.pair import PairDensity
from gpaw.utilities.memory import maxrss
from gpaw.utilities.blas import gemm, rk, czher, mmm
from gpaw.wavefunctions.pw import PWDescriptor
from gpaw.response.pair import PWSymmetryAnalyzer


def frequency_grid(domega0, omega2, omegamax):
    beta = (2**0.5 - 1) * domega0 / omega2
    wmax = int(omegamax / (domega0 + beta * omegamax)) + 2
    w = np.arange(wmax)
    omega_w = w * domega0 / (1 - beta * w)
    return omega_w


class Chi0(PairDensity):
    def __init__(self, calc,
                 frequencies=None, domega0=0.1, omega2=10.0, omegamax=None,
                 ecut=50, hilbert=True, nbands=None,
                 timeordered=False, eta=0.2, ftol=1e-6, threshold=1,
                 real_space_derivatives=False, intraband=True,
                 world=mpi.world, txt=sys.stdout, timer=None,
                 nblocks=1, no_optical_limit=False,
                 keep_occupied_states=False, gate_voltage=None,
                 disable_point_group=False, disable_time_reversal=False,
                 use_more_memory=0, unsymmetrized=True, eshift=None):

        PairDensity.__init__(self, calc, ecut, ftol, threshold,
                             real_space_derivatives, world, txt, timer,
                             nblocks=nblocks,
                             gate_voltage=gate_voltage, eshift=eshift)

        self.eta = eta / Hartree
        self.domega0 = domega0 / Hartree
        self.omega2 = omega2 / Hartree
        self.omegamax = None if omegamax is None else omegamax / Hartree
        self.nbands = nbands or self.calc.wfs.bd.nbands
        self.keep_occupied_states = keep_occupied_states
        self.intraband = intraband
        self.no_optical_limit = no_optical_limit
        self.disable_point_group = disable_point_group
        self.disable_time_reversal = disable_time_reversal
        self.use_more_memory = use_more_memory
        self.unsymmetrized = unsymmetrized

        omax = self.find_maximum_frequency()

        if frequencies is None:
            if self.omegamax is None:
                self.omegamax = omax
            print('Using nonlinear frequency grid from 0 to %.3f eV' %
                  (self.omegamax * Hartree), file=self.fd)
            self.omega_w = frequency_grid(self.domega0, self.omega2,
                                          self.omegamax)
        else:
            self.omega_w = np.asarray(frequencies) / Hartree
            assert not hilbert
        self.hilbert = hilbert
        self.timeordered = bool(timeordered)

        if self.eta == 0.0:
            assert not hilbert
            assert not timeordered
            assert not self.omega_w.real.any()

        # Occupied states:
        wfs = self.calc.wfs
        self.mysKn1n2 = None  # my (s, K, n1, n2) indices
        self.distribute_k_points_and_bands(0, self.nocc2,
                                           kpts=range(wfs.kd.nibzkpts))
        self.mykpts = None

        self.prefactor = 2 / self.vol / wfs.kd.nbzkpts / wfs.nspins

        self.chi0_vv = None  # strength of intraband peak

    def find_maximum_frequency(self):
        self.epsmin = 10000.0
        self.epsmax = -10000.0
        for kpt in self.calc.wfs.kpt_u:
            self.epsmin = min(self.epsmin, kpt.eps_n[0])
            self.epsmax = max(self.epsmax, kpt.eps_n[self.nbands - 1])

        print('Minimum eigenvalue: %10.3f eV' % (self.epsmin * Hartree),
              file=self.fd)
        print('Maximum eigenvalue: %10.3f eV' % (self.epsmax * Hartree),
              file=self.fd)

        return self.epsmax - self.epsmin

    def calculate(self, q_c, spin='all', A_x=None):
        wfs = self.calc.wfs

        if spin == 'all':
            spins = range(wfs.nspins)
        else:
            assert spin in range(wfs.nspins)
            spins = [spin]

        q_c = np.asarray(q_c, dtype=float)
        qd = KPointDescriptor([q_c])
        pd = PWDescriptor(self.ecut, wfs.gd, complex, qd)

        self.print_chi(pd)

        if extra_parameters.get('df_dry_run'):
            print('    Dry run exit', file=self.fd)
            raise SystemExit

        nG = pd.ngmax
        nw = len(self.omega_w)
        mynG = (nG + self.blockcomm.size - 1) // self.blockcomm.size
        self.Ga = self.blockcomm.rank * mynG
        self.Gb = min(self.Ga + mynG, nG)
        assert mynG * (self.blockcomm.size - 1) < nG
        
        if A_x is not None:
            nx = nw * (self.Gb - self.Ga) * nG
            chi0_wGG = A_x[:nx].reshape((nw, self.Gb - self.Ga, nG))
            chi0_wGG[:] = 0.0
        else:
            chi0_wGG = np.zeros((nw, self.Gb - self.Ga, nG), complex)

        if np.allclose(q_c, 0.0):
            chi0_wxvG = np.zeros((len(self.omega_w), 2, 3, nG), complex)
            chi0_wvv = np.zeros((len(self.omega_w), 3, 3), complex)
            self.chi0_vv = np.zeros((3, 3), complex)
        else:
            chi0_wxvG = None
            chi0_wvv = None

        # Do all empty bands:
        m1 = self.nocc1
        m2 = self.nbands
        
        self._calculate(pd, chi0_wGG, chi0_wxvG, chi0_wvv, m1, m2, spins)
        
        return pd, chi0_wGG, chi0_wxvG, chi0_wvv

    @timer('Calculate CHI_0')
    def _calculate(self, pd, chi0_wGG, chi0_wxvG, chi0_wvv, m1, m2, spins):
        # Choose which update method to use
        if self.eta == 0.0:
            update = self.update_hermitian
        elif self.hilbert:
            update = self.update_hilbert
        else:
            update = self.update

        q_c = pd.kd.bzk_kc[0]
        optical_limit = not self.no_optical_limit and np.allclose(q_c, 0.0)
        generator = self.generate_pair_densities

        # Use symmetries
        PWSA = PWSymmetryAnalyzer
        PWSA = PWSA(self.calc.wfs.kd, pd,
                    disable_point_group=self.disable_point_group,
                    disable_time_reversal=self.disable_time_reversal,
                    timer=self.timer, txt=self.fd)

        # If chi's are supplied it
        # is assumed that they are symmetric
        # and we have to divide by the number of
        # symmetries if we are adding
        # the unsymmetric chi
        if self.unsymmetrized:
            nsym = PWSA.how_many_symmetries()
            if nsym > 1:
                chi0_wGG /= nsym
                if chi0_wxvG is not None:
                    chi0_wxvG /= nsym
                if chi0_wvv is not None:
                    chi0_wvv /= nsym

        # Calculate unsymmetrized chi or spectral function
        self.timer.start('Loop')
        for f2_m, df_m, deps_m, n_mG, n_mv, vel_mv in \
            generator(pd, m1, m2, spins, PWSA=PWSA,
                      disable_optical_limit=not optical_limit,
                      intraband=self.intraband,
                      use_more_memory=self.use_more_memory,
                      unsymmetrized=self.unsymmetrized):
            # If the generator returns None for a pair-density
            # then skip updating
            if n_mG is not None:
                update(np.ascontiguousarray(n_mG), deps_m, df_m, chi0_wGG)
            if optical_limit and n_mv is not None:
                self.update_optical_limit(n_mv, deps_m, df_m,
                                          n_mG, chi0_wxvG, chi0_wvv)
            if optical_limit and self.intraband and vel_mv is not None:
                self.update_intraband(f2_m, vel_mv, self.chi0_vv)
        self.timer.stop('Loop')

        # Sum chi
        with self.timer('Sum CHI_0'):
            for chi0_GG in chi0_wGG:
                self.kncomm.sum(chi0_GG)

            if optical_limit:
                self.kncomm.sum(chi0_wxvG)
                self.kncomm.sum(chi0_wvv)
                if self.intraband:
                    self.kncomm.sum(self.chi0_vv)

        print('Memory used: {0:.3f} MB / CPU'.format(maxrss() / 1024**2),
              file=self.fd)

        if (self.eta == 0.0 or self.hilbert) and self.blockcomm.size == 1:
            # Fill in upper/lower triangle also:
            nG = pd.ngmax
            il = np.tril_indices(nG, -1)
            iu = il[::-1]
            if self.hilbert:
                for chi0_GG in chi0_wGG:
                    chi0_GG[il] = chi0_GG[iu].conj()
            else:
                for chi0_GG in chi0_wGG:
                    chi0_GG[iu] = chi0_GG[il].conj()

        if self.hilbert:
            with self.timer('Hilbert transform'):
                ht = HilbertTransform(self.omega_w, self.eta,
                                      self.timeordered)
                ht(chi0_wGG)
                if optical_limit:
                    ht(chi0_wvv)
                    ht(chi0_wxvG)
            print('Hilbert transform done', file=self.fd)

        if optical_limit and self.intraband:  # Add intraband contribution
            omega_w = self.omega_w.copy()
            if omega_w[0] == 0.0:
                omega_w[0] = 1e-14

            chi0_vv = self.chi0_vv
            self.world.broadcast(chi0_vv, 0)

            chi0_wvv += (chi0_vv[np.newaxis] /
                         (omega_w[:, np.newaxis, np.newaxis]
                          + 1j * self.eta)**2)

        if self.unsymmetrized:
            # Carry out symmetrization
            # Redistribute if block par
            tmpchi0_wGG = self.redistribute(chi0_wGG)
            PWSA.symmetrize_wGG(tmpchi0_wGG)
            self.redistribute(tmpchi0_wGG, chi0_wGG)

            if optical_limit:
                PWSA.symmetrize_wxvG(chi0_wxvG)
                PWSA.symmetrize_wvv(chi0_wvv)
                # Since chi_wGG is nonanalytic in the head
                # and wings we have to take care that
                # these are handled correctly. Note that
                # it is important that the wings are overwritten first.
                chi0_wGG[:, :, 0] = chi0_wxvG[:, 1, 0, self.Ga:self.Gb]
                if self.blockcomm.rank == 0:
                    chi0_wGG[:, 0] = chi0_wxvG[:, 0, 0]
                    chi0_wGG[:, 0, 0] = chi0_wvv[:, 0, 0]
                    
        return pd, chi0_wGG, chi0_wxvG, chi0_wvv

    @timer('CHI_0 update')
    def update(self, n_mG, deps_m, df_m, chi0_wGG):
        """Update chi."""

        if self.timeordered:
            deps1_m = deps_m + 1j * self.eta * np.sign(deps_m)
            deps2_m = deps1_m
        else:
            deps1_m = deps_m + 1j * self.eta
            deps2_m = deps_m - 1j * self.eta
        
        for omega, chi0_GG in zip(self.omega_w, chi0_wGG):
            x_m = df_m * (1 / (omega + deps1_m) - 1 / (omega - deps2_m))
            if self.blockcomm.size > 1:
                nx_mG = n_mG[:, self.Ga:self.Gb] * x_m[:, np.newaxis]
            else:
                nx_mG = n_mG * x_m[:, np.newaxis]
            gemm(self.prefactor, n_mG.conj(), np.ascontiguousarray(nx_mG.T),
                 1.0, chi0_GG)

    @timer('CHI_0 hermitian update')
    def update_hermitian(self, n_mG, deps_m, df_m, chi0_wGG):
        """If eta=0 use hermitian update."""
        for w, omega in enumerate(self.omega_w):
            if self.blockcomm.size == 1:
                x_m = (-2 * df_m * deps_m / (omega.imag**2 + deps_m**2))**0.5
                nx_mG = n_mG.conj() * x_m[:, np.newaxis]
                rk(-self.prefactor, nx_mG, 1.0, chi0_wGG[w], 'n')
            else:
                x_m = 2 * df_m * deps_m / (omega.imag**2 + deps_m**2)
                mynx_mG = n_mG[:, self.Ga:self.Gb] * x_m[:, np.newaxis]
                mmm(self.prefactor, mynx_mG, 't', n_mG.conj(), 'n',
                    1.0, chi0_wGG[w])

    @timer('CHI_0 spectral function update')
    def update_hilbert(self, n_mG, deps_m, df_m, chi0_wGG):
        """Update spectral function.

        Updates spectral function A_wGG and saves it to chi0_wGG for
        later hilbert-transform."""

        self.timer.start('prep')
        beta = (2**0.5 - 1) * self.domega0 / self.omega2
        o_m = abs(deps_m)
        w_m = (o_m / (self.domega0 + beta * o_m)).astype(int)
        o1_m = self.omega_w[w_m]
        o2_m = self.omega_w[w_m + 1]
        p_m = self.prefactor * abs(df_m) / (o2_m - o1_m)**2  # XXX abs()?
        p1_m = p_m * (o2_m - o_m)
        p2_m = p_m * (o_m - o1_m)
        self.timer.stop('prep')

        if self.blockcomm.size > 1:
            for p1, p2, n_G, w in zip(p1_m, p2_m, n_mG, w_m):
                myn_G = n_G[self.Ga:self.Gb].reshape((-1, 1))
                gemm(p1, n_G.reshape((-1, 1)), myn_G, 1.0, chi0_wGG[w], 'c')
                gemm(p2, n_G.reshape((-1, 1)), myn_G, 1.0, chi0_wGG[w + 1],
                     'c')
            return

        for p1, p2, n_G, w in zip(p1_m, p2_m, n_mG, w_m):
            czher(p1, n_G.conj(), chi0_wGG[w])
            czher(p2, n_G.conj(), chi0_wGG[w + 1])

    @timer('CHI_0 optical limit update')
    def update_optical_limit(self, n0_mv, deps_m, df_m, n_mG,
                             chi0_wxvG, chi0_wvv):
        """Optical limit update of chi."""

        if self.hilbert:  # Do something special when hilbert transforming
            self.update_optical_limit_hilbert(n0_mv, deps_m, df_m, n_mG,
                                              chi0_wxvG, chi0_wvv)
            return

        if self.timeordered:
            # avoid getting a zero from np.sign():
            deps1_m = deps_m + 1j * self.eta * np.sign(deps_m + 1e-20)
            deps2_m = deps1_m
        else:
            deps1_m = deps_m + 1j * self.eta
            deps2_m = deps_m - 1j * self.eta

        for w, omega in enumerate(self.omega_w):
            x_m = self.prefactor * df_m * (1 / (omega + deps1_m) -
                                           1 / (omega - deps2_m))

            chi0_wvv[w] += np.dot(x_m * n0_mv.T, n0_mv.conj())
            chi0_wxvG[w, 0, :, 1:] += np.dot(x_m * n0_mv.T, n_mG[:, 1:].conj())
            chi0_wxvG[w, 1, :, 1:] += np.dot(x_m * n0_mv.T.conj(), n_mG[:, 1:])

    @timer('CHI_0 optical limit hilbert-update')
    def update_optical_limit_hilbert(self, n0_mv, deps_m, df_m, n_mG,
                                     chi0_wxvG, chi0_wvv):
        """Optical limit update of chi-head and -wings."""

        beta = (2**0.5 - 1) * self.domega0 / self.omega2
        for deps, df, n0_v, n_G in zip(deps_m, df_m, n0_mv, n_mG):
            o = abs(deps)
            w = int(o / (self.domega0 + beta * o))
            if w + 2 > len(self.omega_w):
                break
            o1, o2 = self.omega_w[w:w + 2]
            assert o1 <= o <= o2, (o1, o, o2)

            p = self.prefactor * abs(df) / (o2 - o1)**2  # XXX abs()?
            p1 = p * (o2 - o)
            p2 = p * (o - o1)
            x_vv = np.outer(n0_v, n0_v.conj())
            chi0_wvv[w] += p1 * x_vv
            chi0_wvv[w + 1] += p2 * x_vv
            x_vG = np.outer(n0_v, n_G[1:].conj())
            chi0_wxvG[w, 0, :, 1:] += p1 * x_vG
            chi0_wxvG[w + 1, 0, :, 1:] += p2 * x_vG
            chi0_wxvG[w, 1, :, 1:] += p1 * x_vG.conj()
            chi0_wxvG[w + 1, 1, :, 1:] += p2 * x_vG.conj()

    @timer('CHI_0 intraband update')
    def update_intraband(self, f_m, vel_mv, chi0_vv):
        """Add intraband contributions"""
        assert len(f_m) == len(vel_mv), print(len(f_m), len(vel_mv))

        width = self.calc.occupations.width
        if width == 0.0:
            return

        assert isinstance(self.calc.occupations, FermiDirac)
        dfde_m = - 1. / width * (f_m - f_m**2.0)
        partocc_m = np.abs(dfde_m) > 1e-5
        if not partocc_m.any():
            return

        for dfde, vel_v in zip(dfde_m, vel_mv):
            x_vv = (-self.prefactor * dfde *
                    np.outer(vel_v, vel_v))
            chi0_vv += x_vv
                
    @timer('redist')
    def redistribute(self, in_wGG, out_x=None):
        """Redistribute array.
        
        Switch between two kinds of parallel distributions:
            
        1) parallel over G-vectors (second dimension of in_wGG)
        2) parallel over frequency (first dimension of in_wGG)

        Returns new array using the memory in the 1-d array out_x.
        """
        
        comm = self.blockcomm
        
        if comm.size == 1:
            return in_wGG
            
        nw = len(self.omega_w)
        nG = in_wGG.shape[2]
        mynw = (nw + comm.size - 1) // comm.size
        mynG = (nG + comm.size - 1) // comm.size
        
        bg1 = BlacsGrid(comm, comm.size, 1)
        bg2 = BlacsGrid(comm, 1, comm.size)
        md1 = BlacsDescriptor(bg1, nw, nG**2, mynw, nG**2)
        md2 = BlacsDescriptor(bg2, nw, nG**2, nw, mynG * nG)
        
        if len(in_wGG) == nw:
            mdin = md2
            mdout = md1
        else:
            mdin = md1
            mdout = md2
            
        r = Redistributor(comm, mdin, mdout)
        
        outshape = (mdout.shape[0], mdout.shape[1] // nG, nG)
        if out_x is None:
            out_wGG = np.empty(outshape, complex)
        else:
            out_wGG = out_x[:np.product(outshape)].reshape(outshape)
            
        r.redistribute(in_wGG.reshape(mdin.shape),
                       out_wGG.reshape(mdout.shape))
        
        return out_wGG

    @timer('dist freq')
    def distribute_frequencies(self, chi0_wGG):
        """Distribute frequencies to all cores."""
        
        world = self.world
        comm = self.blockcomm
        
        if world.size == 1:
            return chi0_wGG
            
        nw = len(self.omega_w)
        nG = chi0_wGG.shape[2]
        mynw = (nw + world.size - 1) // world.size
        mynG = (nG + comm.size - 1) // comm.size
  
        wa = min(world.rank * mynw, nw)
        wb = min(wa + mynw, nw)

        if self.blockcomm.size == 1:
            return chi0_wGG[wa:wb].copy()

        if self.kncomm.rank == 0:
            bg1 = BlacsGrid(comm, 1, comm.size)
            in_wGG = chi0_wGG.reshape((nw, -1))
        else:
            bg1 = DryRunBlacsGrid(mpi.serial_comm, 1, 1)
            in_wGG = np.zeros((0, 0), complex)
        md1 = BlacsDescriptor(bg1, nw, nG**2, nw, mynG * nG)
        
        bg2 = BlacsGrid(world, world.size, 1)
        md2 = BlacsDescriptor(bg2, nw, nG**2, mynw, nG**2)
        
        r = Redistributor(world, md1, md2)
        shape = (wb - wa, nG, nG)
        out_wGG = np.empty(shape, complex)
        r.redistribute(in_wGG, out_wGG.reshape((wb - wa, nG**2)))
        
        return out_wGG

    def print_chi(self, pd):
        calc = self.calc
        gd = calc.wfs.gd

        if extra_parameters.get('df_dry_run'):
            from gpaw.mpi import DryRunCommunicator
            size = extra_parameters['df_dry_run']
            world = DryRunCommunicator(size)
        else:
            world = self.world

        print('%s' % ctime(), file=self.fd)
        print('Called response.chi0.calculate with', file=self.fd)

        q_c = pd.kd.bzk_kc[0]
        print('    q_c: [%f, %f, %f]' % (q_c[0], q_c[1], q_c[2]), file=self.fd)

        nw = len(self.omega_w)
        print('    Number of frequency points: %d' % nw, file=self.fd)

        ecut = self.ecut * Hartree
        print('    Planewave cutoff: %f' % ecut, file=self.fd)

        ns = calc.wfs.nspins
        print('    Number of spins: %d' % ns, file=self.fd)

        nbands = self.nbands
        print('    Number of bands: %d' % nbands, file=self.fd)

        nk = calc.wfs.kd.nbzkpts
        print('    Number of kpoints: %d' % nk, file=self.fd)

        nik = calc.wfs.kd.nibzkpts
        print('    Number of irredicible kpoints: %d' % nik, file=self.fd)
        
        ngmax = pd.ngmax
        print('    Number of planewaves: %d' % ngmax, file=self.fd)

        eta = self.eta * Hartree
        print('    Broadening (eta): %f' % eta, file=self.fd)
        
        wsize = world.size
        print('    world.size: %d' % wsize, file=self.fd)

        knsize = self.kncomm.size
        print('    kncomm.size: %d' % knsize, file=self.fd)

        bsize = self.blockcomm.size
        print('    blockcomm.size: %d' % bsize, file=self.fd)
        
        nocc = self.nocc1
        print('    Number of completely occupied states: %d'
              % nocc, file=self.fd)
        
        npocc = self.nocc2
        print('    Number of partially occupied states: %d'
              % npocc, file=self.fd)

        keep = self.keep_occupied_states
        print('    Keep occupied states: %s' % keep, file=self.fd)

        print('', file=self.fd)
        print('    Memory estimate of potentially large arrays:', file=self.fd)

        chisize = nw * pd.ngmax**2 * 16. / 1024**2
        print('        chi0_wGG: %f M / cpu' % chisize, file=self.fd)

        ngridpoints = gd.N_c[0] * gd.N_c[1] * gd.N_c[2]

        if self.keep_occupied_states:
            nstat = (ns * nk * npocc + world.size - 1) // world.size
        else:
            nstat = (ns * npocc + world.size - 1) // world.size

        occsize = nstat * ngridpoints * 16. / 1024**2
        print('        Occupied states: %f M / cpu' % occsize,
              file=self.fd)

        print('        Memory usage before allocation: %f M / cpu'
              % (maxrss() / 1024**2), file=self.fd)

        print('', file=self.fd)


class HilbertTransform:
    def __init__(self, omega_w, eta, timeordered=False, gw=False,
                 blocksize=500):
        """Analytic Hilbert transformation using linear interpolation.

        Hilbert transform::

           oo
          /           1                1
          |dw' (-------------- - --------------) S(w').
          /     w - w' + i eta   w + w' + i eta
          0

        With timeordered=True, you get::

           oo
          /           1                1
          |dw' (-------------- - --------------) S(w').
          /     w - w' - i eta   w + w' + i eta
          0

        With gw=True, you get::

           oo
          /           1                1
          |dw' (-------------- + --------------) S(w').
          /     w - w' + i eta   w + w' + i eta
          0

        """

        self.blocksize = blocksize

        if timeordered:
            self.H_ww = self.H(omega_w, -eta) + self.H(omega_w, -eta, -1)
        elif gw:
            self.H_ww = self.H(omega_w, eta) - self.H(omega_w, -eta, -1)
        else:
            self.H_ww = self.H(omega_w, eta) + self.H(omega_w, -eta, -1)

    def H(self, o_w, eta, sign=1):
        """Calculate transformation matrix.

        With s=sign (+1 or -1)::

                        oo
                       /       dw'
          X (w, eta) = | ---------------- S(w').
           s           / s w - w' + i eta
                       0

        Returns H_ij so that X_i = np.dot(H_ij, S_j), where::

            X_i = X (omega_w[i]) and S_j = S(omega_w[j])
                   s
        """

        nw = len(o_w)
        H_ij = np.zeros((nw, nw), complex)
        do_j = o_w[1:] - o_w[:-1]
        for i, o in enumerate(o_w):
            d_j = o_w - o * sign
            y_j = 1j * np.arctan(d_j / eta) + 0.5 * np.log(d_j**2 + eta**2)
            y_j = (y_j[1:] - y_j[:-1]) / do_j
            H_ij[i, :-1] = 1 - (d_j[1:] - 1j * eta) * y_j
            H_ij[i, 1:] -= 1 - (d_j[:-1] - 1j * eta) * y_j
        return H_ij

    def __call__(self, S_wx):
        """Inplace transform"""
        B_wx = S_wx.reshape((len(S_wx), -1))
        nw, nx = B_wx.shape
        tmp_wx = np.zeros((nw, min(nx, self.blocksize)), complex)
        for x in range(0, nx, self.blocksize):
            b_wx = B_wx[:, x:x + self.blocksize]
            c_wx = tmp_wx[:, :b_wx.shape[1]]
            gemm(1.0, b_wx, self.H_ww, 0.0, c_wx)
            b_wx[:] = c_wx


if __name__ == '__main__':
    do = 0.025
    eta = 0.1
    omega_w = frequency_grid(do, 10.0, 3)
    print(len(omega_w))
    X_w = omega_w * 0j
    Xt_w = omega_w * 0j
    Xh_w = omega_w * 0j
    for o in -np.linspace(2.5, 2.9, 10):
        X_w += (1 / (omega_w + o + 1j * eta) -
                1 / (omega_w - o + 1j * eta)) / o**2
        Xt_w += (1 / (omega_w + o - 1j * eta) -
                 1 / (omega_w - o + 1j * eta)) / o**2
        w = int(-o / do / (1 + 3 * -o / 10))
        o1, o2 = omega_w[w:w + 2]
        assert o1 - 1e-12 <= -o <= o2 + 1e-12, (o1, -o, o2)
        p = 1 / (o2 - o1)**2 / o**2
        Xh_w[w] += p * (o2 - -o)
        Xh_w[w + 1] += p * (-o - o1)

    ht = HilbertTransform(omega_w, eta, 1)
    ht(Xh_w)

    import matplotlib.pyplot as plt
    plt.plot(omega_w, X_w.imag, label='ImX')
    plt.plot(omega_w, X_w.real, label='ReX')
    plt.plot(omega_w, Xt_w.imag, label='ImXt')
    plt.plot(omega_w, Xt_w.real, label='ReXt')
    plt.plot(omega_w, Xh_w.imag, label='ImXh')
    plt.plot(omega_w, Xh_w.real, label='ReXh')
    plt.legend()
    plt.show()



import numpy as np
from gpaw.mpi import world
from gpaw.transport.tools import fermidistribution, gather_ndarray_dict
from ase.parallel import parprint, paropen

'''The Contour class is used to get the integral contour parallelly and control
   the necessary parameters.
 
   To get a Keldysh Green Function, one need to to the integral like this:
 
                      /                          
                      |                         ---- 
      D=    1.j/(2*pi)| (Gr(Z) - Ga(Z)f(Z)dZ -  \   (Gr(E) - Ga(E)) * kt 
                      /                         /___
                   1, 2, 3                        4
                             /
                             |
                   + 1/(2*pi)| G<(E)dE
                             /
                             5
   The Path indexed by (1,2,3,4,5,6) looks like below:   
 
                              ^   
            |\                |
            | \               |
            |  \ 2            |
            |   \             |
          1 |    \            |
            |     \    3      |        6
            |      \---------------------
            |                 |4                 5
            |    --------------------------------------
           --------------------------------------------->
 
 
    Hierarchy:
       NID(node ID) is the most important information of the energy node.
       With it you can get the corresponding energy and weight.
 
    To evaluate a function(named f) integral on a region[c-h, c+h],
    first, we use the Gauss-Lobatto formula to get a evaluation Q1
      / c+h
      |
      |  f(x) dx  = 1/6*f(c-h) + 5/6*f(c-h/sqrt(5)) + 5/6*f(c+h/sqrt(5)) + 1/6*f(c+h) = Q1
      |
      / c-h
 
    then, we use the cooresponding Kronrod formula to get another evalution Q2
      / c+h
      |
      |  f(x) dx  = 77/1470*f(c-h) + 432/1470*f(c-sqrt(2/3)*h) + 625/1470*f(c-h/sqrt(5)) +
      |
      / c-h
           672/1470*f(c) + 625/1470*f(c+h/sqrt(5)) + 432/1470*f(c+sqrt(2/3)*h) + 77/1470*f(c+h) = Q2

    we compare the two results, if the difference is less than tolerance, then divide
    the region into six parts, and for each of them, do the two evaluations until the difference
    less than the tolerance.''' 

class EnergyNode:
    def __init__(self, type, nlead):
        self.type = type
        self.num = 0
        self.lead_num = nlead
        self.energy = []
        self.weight = []
        self.nres = 0
        self.sigma = []
        for i in range(nlead):
            self.sigma.append([])
        if type == 'eq':
            self.fermi_factor = []
        elif type == 'ne':
            self.fermi_factor = []
            for i in range(nlead):
                self.fermi_factor.append([[], []])
        else:
            raise TypeError('unkown EnergyNode type')

    def add(self, elist, wlist, flist, siglist):
        self.num += len(elist)
        self.energy += elist
        self.weight += wlist
        if self.type == 'eq':
            self.fermi_factor += flist
        elif self.type == 'ne':
            for i in range(self.lead_num):
                for j in [0, 1]:
                    self.fermi_factor[i][j] += flist[i][j]
        else:
            raise TypeError('unkown EnergyNode type')
        for i in range(self.lead_num):
            self.sigma[i] += siglist[i]

    def set_nres(self, nres):
        self.nres = nres
        
class Path:
    poles_num = 4
    #int_step = 0.02
    bias_step = 0.1
    zone_sample = np.array([0, 0.55278640450004, 1.44721359549996, 2.0,
                            0.0]) / 2.
    zone_length = np.array([2., 0.55278640450004, 0.89442719099992007,
                                                     0.55278640450004]) / 2.0
    sample = np.array([0, 0.18350341907227,   0.55278640450004,   1.0,
         1.44721359549996,   1.81649658092773, 2.0]) / 2.
    zone_weight  = np.array([2., 0.55278640450004, 0.89442719099992007,
                                                     0.55278640450004]) / 2.0
    weights0 = np.array([6.0, 1.0, 0.0, 5.0, 0.0, 5.0, 0.0, 1.0]) / 6.0
    weights1 = np.array([1470., 77.0, 432.0, 625.0, 672.0, 625.0,
                          432.0, 77.0]) / 1470.
    weights2 = [3.0 / 8, 7.0 / 6, 23.0 / 24] 
    weights3 = [23.0 / 24, 7.0 / 6, 3.0 / 8]
    #these two parameters used to build selfenergy database are obselete
    bias_window_begin = -3
    bias_window_end = 3
    #------------
    def __init__(self, begin, end, index, maxdepth=7, type='Gaussian',
                 kt=0.1, int_step=0.02):
        self.begin = begin
        self.end = end
        self.type = type
        self.index = index
        self.maxdepth = maxdepth
        self.nids = []
        self.energies = []
        self.functions = []
        self.ses = []
        self.num = 10 ** self.maxdepth
        self.full_nodes = False
        self.kt = kt
        self.int_step = int_step
        self.initialize()

    def initialize(self):
        if self.type == 'linear':
            self.ne = int(max(np.ceil(np.abs((self.end - self.begin)
                                                  / self.int_step)) + 1, 6))
            self.int_step = np.abs(self.end - self.begin) / (self.ne - 1)
        
    def get_flags(self, num, path_flag=False):
        flags = []
        if self.type == 'Gaussian':
            if path_flag:
                assert num < self.num * 10
            else:
                assert num < self.num                
            digits = self.maxdepth
            
        elif self.type == 'linear':
            digits = int(np.ceil(np.log10(self.ne)))            
            
        elif self.type == 'poles':
            if self.full_nodes:
                nids_num = int((self.bias_window_end -
                                self.bias_window_begin) //
                                  self.bias_step + 1)                
                digits = int(np.ceil(np.log10(nids_num))) 
            elif path_flag:
                digits = 2
            else:
                digits = 3

        if path_flag:
            digits += 1

        for i in range(digits):
            unit = 10 ** (digits - i)
            digit = (num - (num // unit) * unit) // (unit // 10)
            flags.append(digit)            
        return np.array(flags)
        
    def get_full_nids(self):
        self.full_nodes = True
        self.nids = []
        self.energies = []
        depth = self.maxdepth    
        if self.type == 'Gaussian':
            base = self.num * self.index
            assert depth < 7
            dimension = [3] * (depth - 1) + [6]
            for i in range(self.num):
                flags = self.get_flags(i)
                this_node = True
                for j in range(depth):
                    if flags[j] >= dimension[j]:
                        this_node = False
                    else:
                        flags[j] += 1
                if this_node:
                    new_i = np.sum(flags * 10 ** np.arange(len(flags) - 1,
                                                           -1, -1))
                    self.nids.append(new_i + base)
                    self.energies.append(self.get_energy(flags))
        
        elif self.type == 'linear':
            num = self.ne
            vector = (self.end - self.begin) / (num - 1)
            digits = int(np.ceil(np.log10(num)))
            base = self.index * 10 ** digits
            for i in range(num):
                self.nids.append(i + base)
                self.energies.append(self.begin + i * self.int_step * vector)
        
        elif self.type == 'poles':
            num = int((self.bias_window_end - self.bias_window_begin) //
                                                      self.bias_step + 1)
            real_energies = np.linspace(self.bias_window_begin,
                                        self.bias_window_end, num)
            digits = int(np.ceil(np.log10(num * self.poles_num)))
            base = self.index * 10 ** digits
            for i in range(num):
                for k in range(self.poles_num):
                    self.nids.append(k + i * 10 + base)
                    self.energies.append(real_energies[i] +
                                        (2 * k + 1) * np.pi * self.kt * 1.j)
        else:
            raise RuntimeWarning('Wrong path type %s' % self.type)
            
    def get_new_nids(self, zone, depth=0):
        assert depth < 7
        nids = []
        if self.type == 'Gaussian':
            base = zone * 10 ** (self.maxdepth - depth)
            for i in range(2, 7):
                nids.append(base + i)
        elif self.type == 'linear':
            assert depth == 0
            nids_num = self.ne
            digits = int(np.ceil(np.log10(nids_num)))
            base = self.index * 10 ** digits
            for i in range(1, nids_num + 1):
                nids.append(i + base)
        elif self.type == 'poles':
            assert depth == 0
            base = zone * 100
            nids0 = base + 10 + np.arange(self.poles_num) + 1
            nids1 = base + 20 + np.arange(self.poles_num) + 1
            nids = np.append(nids0, nids1).tolist()
        else:
            raise RuntimeError('Wrong Path Type % s' % self.type)
        return nids
    
    def add_node(self, nid, energy, function, se):
        self.nids.append(nid)
        self.functions.append(function)
        self.energies.append(energy)
        self.ses.append(se)
  
    def get_ends_nids(self):
        nids = []
        num = self.num
        if self.index in [1, 2, 3, 6]:
            nids.append(self.index * num + 1)
        if self.index in [3, 6]:
            nids.append(self.index * num + 7)
        return nids
       
    def get_energy(self, flags):
        if self.type == 'Gaussian':
            pps = np.append([1], self.zone_length[np.array(flags[:-1])])
            lls = []
            for i in range(self.maxdepth):
                lls.append(np.product(pps[:i+1]))
            ss = np.append(self.zone_sample[np.array(flags[:-1])- 1],
                           self.sample[flags[-1] - 1])
            energy = np.sum(ss * lls) * (self.end - self.begin) + self.begin
            
        elif self.type == 'linear':
            ind = self.tonid(flags[1:])
            energy = np.linspace(self.begin, self.end, self.ne)[ind - 1]
       
        elif self.type == 'poles':
            if self.full_nodes:
                num = int((self.bias_window_end - self.bias_window_begin) //
                                                      self.bias_step + 1)
                real_energies = np.linspace(self.bias_window_begin,
                                        self.bias_window_end, num)
                tens = np.arange(len(flags[:-1]) - 1, -1) ** 10
                line_index = np.sum(flags[:-1] * tens)
                energy = real_energies[line_index] + (2 * flags[-1] -
                                                 1) * np.pi * self.kt * 1.j
            else:
                lines = [self.begin, self.end]
                energy = lines[flags[0] - 1] + (2 * flags[-1] -
                                                 1) * np.pi * self.kt * 1.j
        return energy
    
    def get_weight(self, flags):
        if self.type == 'Gaussian':
            wei0 = (self.end - self.begin) / 2.
            for i in range(self.maxdepth - 1):
                wei0 *= self.zone_weight[flags[i + 1]]
            wei1 = wei0
            wei0 *= self.weights0[flags[-1]]
            wei1 *= self.weights1[flags[-1]]
            return wei0, wei1
        elif self.type == 'linear':
            wei = self.weights2 + (self.ne - 6) * 1 + self.weights3
            ind = self.tonid(flags[1:]) - 1
            return wei[ind]
        elif self.type == 'poles':
            return 1

    def tonid(self, flags):
        dim = len(flags)
        nid = 0
        for i in range(dim):
            nid += flags[i] * 10 ** (dim - i - 1)
        return nid

class Contour:
    # see the file description of contour
    ne_err = 1e-4
    calcutype = ['eqInt', 'eqInt', 'eqInt', 'resInt', 'neInt', 'locInt']
    def __init__(self, kt, fermi, bias, maxdepth=7, comm=None, neint='linear',
                  plot_eta=1e-4, neintstep=0.02, eqinttol=1e-4, eta=0.01,
                  min_energy=-700, plot_energy_range=[-5.,5.],
                  plot_energy_point_num=201):
        self.kt = kt
        self.nkt = 8 * self.kt 
        self.dkt = 8 * np.pi * self.kt       
        self.fermi = fermi[0]
        self.bias = bias
        self.neint = neint
        self.leadfermi = []
        for i in range(len(bias)):
            self.leadfermi.append(fermi[i] + bias[i])
        self.minfermi = min(self.leadfermi)
        self.maxfermi = max(self.leadfermi)
        self.plot_eta = plot_eta
        self.eta = eta
        self.dtype = complex
        self.comm = comm
        if self.comm is None:
            self.comm = world
        self.maxdepth = maxdepth
        self.num = 10 ** (self.maxdepth - 1)
        self.converged_zones = []
        self.total_sum = 0
        self.plot_path = None
        self.neintstep = neintstep
        self.eqinttol = eqinttol
        self.min_energy = min_energy
        self.eq_err = eqinttol
        self.plot_energy_range = plot_energy_range
        self.plot_energy_point_num = plot_energy_point_num
        
    def get_dense_contour(self):
        self.paths = []
        depth = self.maxdepth
        self.paths.append(Path(self.min_energy, self.min_energy + 20. * 1.j,
                               1, 1, type='Gaussian', kt=self.kt))
        self.paths.append(Path(self.min_energy + 20. * 1.j, -4. + np.pi * 1.j,
                               2, 3, type='Gaussian', kt=self.kt))
        self.paths.append(Path(-4. + np.pi * 1.j, 4. + np.pi * 1.j, 3, depth,
                              type='linear', kt=self.kt))
        self.paths.append(Path(-3., 3., 4, depth, type='poles', kt=self.kt))
        self.paths.append(Path(-5. + self.eta * 1.j, 5. + self.eta * 1.j, 5,
                              depth, type='linear', kt=self.kt)) 
        for i in range(5):
            self.paths[i].get_full_nids()

    def get_optimized_contour(self, tp):
        tp.log('Contour.get_optimized_contour()')
        self.paths = []
        depth = self.maxdepth
        self.paths.append(Path(self.min_energy + self.minfermi,
                               self.min_energy + self.minfermi +
                               (10. + self.dkt) * 1.j,
                                1, depth,
                               type='Gaussian', kt=self.kt))
        
        self.paths.append(Path(self.min_energy + self.minfermi +
                                (10 + self.dkt) * 1.j,
                                self.minfermi - self.nkt + self.dkt * 1.j,
                                2, depth,
                                type='Gaussian', kt=self.kt))
        
        self.paths.append(Path(self.minfermi - self.nkt + self.dkt * 1.j,
                              self.minfermi + self.nkt + self.dkt * 1.j,
                              3, depth,
                              type='Gaussian', kt=self.kt))
        
        self.paths.append(Path(self.minfermi,
                               self.maxfermi,
                               4, depth,
                               type='poles', kt=self.kt))
        
        self.paths.append(Path(self.minfermi - self.nkt + self.eta * 1.j,
                              self.maxfermi + self.nkt + self.eta * 1.j,
                               5, depth,
                               type=self.neint, kt=self.kt,
                               int_step=self.neintstep))
        
        self.paths.append(Path(self.minfermi - self.nkt + self.dkt * 1.j,
                              self.maxfermi + self.nkt + self.dkt * 1.j,
                               6, depth,
                               type='Gaussian', kt=self.kt))
        
        zones = np.arange(1, 7)
        depth = 0
        converge = False
        while not converge and depth < self.maxdepth:
            nids, path_indices = self.collect(zones, depth)
            loc_nids, loc_path_indices = self.distribute(nids, path_indices)
            self.calculate(tp, loc_nids, loc_path_indices)
            if depth == 0:
                self.joint(zones)
            converge, zones = self.check_convergence(zones, depth, tp.nbmol)
            depth += 1
            self.transfer(zones, depth)

    def get_plot_path(self, ex=False):
        if ex:
            limit = 6.5
        else:    
            limits = self.plot_energy_range
        
        if self.plot_path is None:
            self.plot_path = Path(limits[0] + self.fermi + self.plot_eta * 1.j,
                              limits[1] + self.fermi + self.plot_eta * 1.j,
                               7, 1,
                               type='linear', kt=self.kt)
            path = self.plot_path  
            if ex:
                path.ne = 261
            else:    
                path.ne = self.plot_energy_point_num
            path.int_step = (limits[1] - limits[0]) / (
                                               self.plot_energy_point_num - 1)
           
            digits = int(np.ceil(np.log10(path.ne)))
            base = path.index * 10 ** digits
            energies = np.linspace(path.begin, path.end, path.ne)
            weights = path.weights2 + [1] * (path.ne - 6) + path.weights3
            weights = np.array(weights) * path.int_step
            nids = np.arange(path.ne) + base + 1

            loc_nids = np.array_split(nids, self.comm.size)[self.comm.rank]
            loc_energies = np.array_split(energies,
                                          self.comm.size)[self.comm.rank]
            loc_weights = np.array_split(weights,
                                         self.comm.size)[self.comm.rank]
            path.my_nids = loc_nids
            path.my_energies = loc_energies
            path.my_weights = loc_weights
            path.weights = weights
            path.energies = energies
            path.nids = nids
        return self.plot_path
           
    def collect(self, zones, depth):
        nids = []
        path_indices = []
        for zone in zones:
            path_index = zone // (10 ** depth) - 1
            new_nids = self.paths[path_index].get_new_nids(zone, depth)
            nids += new_nids
            path_indices += [path_index] * len(new_nids)
            
            if depth == 0:
                new_nids = self.paths[path_index].get_ends_nids()
                nids += new_nids
                path_indices += [path_index] * len(new_nids)
        return nids, path_indices
    
    def distribute(self, nids, path_indices):
        loc_nids = np.array_split(nids, self.comm.size)
        loc_path_indices = np.array_split(path_indices, self.comm.size)
        return loc_nids[self.comm.rank], loc_path_indices[self.comm.rank]
    
    def calculate(self, tp, loc_nids, path_indices):
        tp.log('Contour.calculate()')
        for nid, path_index in zip(loc_nids, path_indices):
            exp10 = int(np.floor(np.log10(nid)))
            flags = self.paths[path_index].get_flags(nid, True)
            energy = self.paths[path_index].get_energy(flags[1:])
            if  path_index in [0, 1, 2, 5]:
                calcutype = self.calcutype[path_index]
                green_function, se = tp.calgfunc(energy, calcutype,
                                                      'new')
                self.paths[path_index].functions.append(np.diag(
                                                          green_function[0]))
                self.paths[path_index].energies.append(energy)
                self.paths[path_index].ses.append(se)
                self.paths[path_index].nids.append(nid)
                
    def joint(self, zones):
        my_zones = np.array_split(zones, self.comm.size)[self.comm.rank]
        my_info_dict = {}
        num = 0
        for zone in my_zones:
            if zone in [3, 4, 5, 6]:
                pass
            else:
                path_index = zone - 1
                order = 10 ** self.maxdepth
                base = zone * order
                nid = base + 7
                link_nid = nid + order - 6
                link_path_index = link_nid // (10 ** self.maxdepth) - 1
                flag = str(self.comm.rank) + '_' + str(num)
                my_info_dict[flag] = np.array([nid, link_nid,
                                           path_index, link_path_index], int)
                num += 1
                  
        info_dict = gather_ndarray_dict(my_info_dict, self.comm,
                                                             broadcast=True)
        for name in info_dict:
            nid, link_nid, path_index, link_path_index = info_dict[name]
            rank = self.get_rank(link_path_index, link_nid)
            
            if self.comm.rank == rank:
                link_path = self.paths[link_path_index]                
                index = link_path.nids.index(link_nid)
                function = link_path.functions[index]
                se = link_path.ses[index]
                energy = link_path.energies[index]
                self.paths[path_index].add_node(nid, energy, function, se)

    def transfer(self, zones, depth):
        my_zones = np.array_split(zones, self.comm.size)[self.comm.rank]
        my_info_dict = {}
        num = 0
        maximum = 100000
        name_flags = []
        for zone in my_zones:
            path_index = zone // (10 ** depth) - 1
            order = 10 ** (self.maxdepth - depth)
            base = zone * order
            node_index = zone % 10
            nid = zone * order + 1

            link_nid = (zone - node_index) * order + 2 * node_index - 1
            flag = str((self.comm.rank  + 1) * maximum + num)                
            my_info_dict[flag] = np.array([nid, link_nid, path_index], int)
            num += 1
                
            nid = zone * order + 7
            link_nid = (zone - node_index) * order + 2 * node_index + 1
            flag = str((self.comm.rank + 1) * maximum + num)
            my_info_dict[flag] = np.array([nid, link_nid, path_index], int)
            num += 1                
            
        info_dict = gather_ndarray_dict(my_info_dict, self.comm,
                                        broadcast=True)
        for name in info_dict:
            name_flags.append(eval(name))
        name_flags = np.sort(name_flags)
        #sort is necessrary because sometime the dict sequency is different for
        #different processors!!

        for name_flag in name_flags:
            nid, link_nid, path_index = info_dict[str(name_flag)]
            rank = self.get_rank(path_index, link_nid)
            if self.comm.rank == rank:
                path = self.paths[path_index]                
                index = path.nids.index(link_nid)
                function = path.functions[index]
                energy = path.energies[index]
                se = path.ses[index]
                self.paths[path_index].add_node(nid, energy, function, se)
            
    def get_rank(self, path_index, nid):
        info_array = np.zeros([self.comm.size], int)
        if nid in self.paths[path_index].nids:
            info_array[self.comm.rank] = 1
        self.comm.sum(info_array)
        assert np.sum(info_array) == 1
        return np.argmax(info_array)
       
    def check_convergence(self, zones, depth, nbmol):
        new_zones = []
        converged = True
        errs = [0]
        err = 0
        for zone in zones:
            indices = []
            nids = []
            if zone in [4, 5]:
                pass
            else:
                order = 10 ** (self.maxdepth - depth)
                base = zone * order
                original_nids = np.arange(1, 8) + base
                path_index = zone // 10 ** depth - 1
                gr_sum0 = np.zeros([1, nbmol, nbmol], dtype=self.dtype)
                gr_sum1 = np.zeros([1, nbmol, nbmol], dtype=self.dtype)
                path = self.paths[path_index]            
                for nid in original_nids:
                    if nid in path.nids:
                        flags = path.get_flags(nid, True)
                        weight0, weight1 = path.get_weight(flags)
                        index = path.nids.index(nid)
                        indices.append(index)
                        nids.append(nid)
                        gr_sum0 += path.functions[index] * weight0
                        gr_sum1 += path.functions[index] * weight1
                self.comm.sum(gr_sum0)
                self.comm.sum(gr_sum1)
                err = np.max(abs(gr_sum0 - gr_sum1))
                if err > self.eq_err:
                    converged = False
                    new_zones += range(zone * 10 + 1, zone * 10 + 4)
                    errs.append(err)
            if err < self.eq_err:
                self.converged_zones.append(zone)
                for ind, nid in zip(indices, nids):
                    digit = nid % 10
                    if digit > 1 and digit < 7:            
                        path.functions[ind] = None
        return converged, new_zones        

    def sort_contour(self, nbmol, cal_den=False):
        weights = []
        energies = []
        nids = []
        ses = []
        for zone in self.converged_zones:
            if zone in [4, 5]:
                if cal_den:
                    den_ne = np.zeros([nbmol, nbmol], dtype=self.dtype)
                else:    
                    pass
            else:
                depth = int(np.floor(np.log10(zone)))
                order = 10 ** (self.maxdepth - depth)
                base = zone * order
                original_nids = np.arange(1, 8, 2) + base
                path_index = zone // 10 ** depth - 1
                if cal_den:
                    den_eq = np.zeros([nbmol, nbmol], dtype=self.dtype)
                path = self.paths[path_index]            
                for nid in original_nids:
                    if nid in path.nids:
                        flags = path.get_flags(nid, True)
                        weight0, weight1 = path.get_weight(flags)
                        energy = path.get_energy(flags[1:])   
                        index = path.nids.index(nid)
                        se = path.ses[index]
                        weights.append(weight0)
                        energies.append(energy)
                        nids.append(nid)
                        ses.append(se)
                        if cal_den:
                            den_eq += path.functions[index] * weight0
                if cal_den:
                    self.comm.sum(den_eq)
                    self.comm.sum(den_ne)
        
        seq = np.argsort(nids)
        del_seq = []
        tol = 1e-9
        for i, j in zip(seq[:-1], seq[1:]):
            if np.abs(energies[i] - energies[j]) < tol:
                weights[j] += weights[i]     
                del_seq.append(i)
    
        del_seq.sort()
        del_seq.reverse()
        for i in del_seq:
            del nids[i]
            del energies[i]
            del weights[i]
            del ses[i]
        del self.converged_zones[:]
        
        if cal_den:
            return nids, energies, weights, ses, (den_eq + den_ne)
        else:
            return nids, energies, weights, ses

    def release(self):
        for path in self.paths:
            for function in path.functions:
                function = None
            for se in path.ses:
                se = None
       
    def distribute_nodes(self, path_index):
        path = self.paths[path_index]
        if path.type == 'linear':
            digits = int(np.ceil(np.log10(path.ne)))
            base = path.index * 10 ** digits
            energies = np.linspace(path.begin, path.end, path.ne)
            weights = path.weights2 + [1] * (path.ne - 6) + path.weights3
            weights = np.array(weights) * path.int_step
            nids = np.arange(path.ne) + base + 1
        
        elif path.type == 'poles':
            base = path.index * 100
            nids0 = base + 10 + np.arange(path.poles_num) + 1
            nids1 = base + 20 + np.arange(path.poles_num) + 1
            nids = np.append(nids0, nids1)
            energies0 = path.begin + (np.arange(path.poles_num) * 2
                                                        - 1) * np.pi * 1.j
            energies1 = path.end + (np.arange(path.poles_num) * 2
                                                        - 1) * np.pi * 1.j
            weights0 = [-1] * path.poles_num
            weights1 = [1] * path.poles_num
            weights = np.append(weights0, weights1)
        
        loc_nids = np.array_split(nids, self.comm.size)[self.comm.rank]
        loc_energies = np.array_split(energies,
                                         self.comm.size)[self.comm.rank]
        loc_weights = np.array_split(weights, self.comm.size)[self.comm.rank]
        return loc_nids, loc_energies, loc_weights





import numpy as np

from ase.parallel import parprint
from ase.units import Bohr

from gpaw import debug
from gpaw.analyse.observers import Observer
from gpaw.transformers import Transformer
from gpaw.lfc import BasisFunctions
from gpaw.utilities import unpack2, is_contiguous

from gpaw.inducedfield.inducedfield_base import BaseInducedField, sendreceive_dict


class FDTDInducedField(BaseInducedField, Observer):
    """Induced field class for FDTD.
    
    Attributes (see also ``BaseInducedField``):
    -------------------------------------------
    time: float
        Current time
    Fn_wsG: ndarray (complex)
        Fourier transform of induced polarization charge density
    n0t_sG: ndarray (float)
        Ground state charge density
    FD_awsp: dict of ndarray (complex)
        Fourier transform of induced D_asp
    D0_asp: dict of ndarray (float)
        Ground state D_asp
    """
    
    def __init__(self, filename=None, paw=None, ws='all',
                  frequencies=None, folding='Gauss', width=0.08,
                  interval=1, restart_file=None
                  ):
        """
        Parameters (see also ``BaseInducedField``):
        -------------------------------------------
        paw: TDDFT object
            TDDFT object for time propagation
        width: float
            Width in eV for the Gaussian (sigma) or Lorentzian (eta) folding
            Gaussian   = exp(- (1/2) * sigma^2 * t^2)
            Lorentzian = exp(- eta * t)
        interval: int
            Number of timesteps between calls (used when attaching)
        restart_file: string
            Name of the restart file
        """

        Observer.__init__(self, interval)
        # From observer:
        # self.niter
        # self.interval
        
        # Restart file
        self.restart_file = restart_file
        
        # These are allocated in allocate()
        self.Fn_wsG = None
        self.n0_G = None


        self.readwritemode_str_to_list = \
        {'': ['Fn', 'n0', 'FD', 'atoms'],
         'all': ['Fn', 'n0', 'FD',
                 'Frho', 'Fphi', 'Fef', 'Ffe', 'eps0', 'atoms'],
         'field': ['Frho', 'Fphi', 'Fef', 'Ffe', 'eps0', 'atoms']}

        BaseInducedField.__init__(self, filename, paw, ws,
                                  frequencies, folding, width)
        
    def initialize(self, paw, allocate=True):
        BaseInducedField.initialize(self, paw, allocate)
        
        # FDTD replacements and overwrites
        self.fdtd   = paw.hamiltonian.poisson
        #self.gd     = self.fdtd.cl.gd.refine()
        self.gd     = self.fdtd.cl.gd
               
        assert hasattr(paw, 'time') and hasattr(paw, 'niter'), 'Use TDDFT!'
        self.time = paw.time
        self.niter = paw.niter
        
        # TODO: remove this requirement
        assert np.count_nonzero(paw.kick_strength) > 0, \
        'Apply absorption kick before %s' % self.__class__.__name__
        
        # Background electric field
        self.Fbgef_v = paw.kick_strength

        # Attach to PAW-type object
        paw.attach(self, self.interval)
        # TODO: write more details (folding, freqs, etc)
        parprint('%s: Attached ' % self.__class__.__name__)

    def set_folding(self, folding, width):
        BaseInducedField.set_folding(self, folding, width)
        
        if self.folding is None:
            self.envelope = lambda t: 1.0
        else:
            if self.folding == 'Gauss':
                self.envelope = lambda t: np.exp(- 0.5 * self.width**2 * t**2)
            elif self.folding == 'Lorentz':
                self.envelope = lambda t: np.exp(- self.width * t)
            else:
                raise RuntimeError('unknown folding "' + self.folding + '"')
        
    def allocate(self):
        if not self.allocated:
            
            # Ground state charge density
            self.n0_G = (-1.0) * self.paw.hamiltonian.poisson.classical_material.sign * \
                                 self.paw.hamiltonian.poisson.classical_material.charge_density.copy()
            
            # Fourier transformed charge density
            self.Fn_wsG = self.paw.hamiltonian.poisson.cl.gd.zeros((self.nw, self.nspins),
                                                                   dtype=self.dtype)
    
            self.allocated = True
            
        if debug:
            assert is_contiguous(self.Ft_wG, self.dtype)

    def deallocate(self):
        BaseInducedField.deallocate(self)
        self.n0t_G = None
        self.Fnt_wG = None
        
    def update(self):
        # Update time
        self.time = self.paw.time
        time_step = self.paw.time_step

        # Complex exponential with envelope
        f_w = np.exp(1.0j * self.omega_w * self.time) * \
              self.envelope(self.time) * time_step

        # Time-dependent quantities
        n_G = (-1.0) * self.fdtd.classical_material.charge_density * self.fdtd.classical_material.sign

        # Update Fourier transforms
        for w in range(self.nw):
            self.Fn_wsG[w] += (n_G - self.n0_G) * f_w[w]

        # Restart file
        if self.restart_file is not None and \
           self.niter % self.paw.dump_interval == 0:
            self.write(self.restart_file)
            parprint('%s: Wrote restart file %s' % (self.__class__.__name__, self.restart_file))
    
        
    def get_induced_density(self, from_density, gridrefinement):
        #Frho_wg = -self.Fn_wG.sum(axis=1).copy()
        #Fn_wG_global = -self.gd.collect(self.Fn_wG).sum(axis=1)
        #if self.gd.comm.rank==0:
        
        Frho_wg = []
        for w in range(self.nw):
            if self.gd == self.fdtd.cl.gd:
                Frho_wg.append(self.Fn_wsG[w].sum(axis=0))
            else:
                Frho_wg.append(Transformer(self.fdtd.cl.gd,
                                           self.gd,
                                           self.stencil,
                                           dtype=self.dtype).apply(self.Fn_wsG[w].sum(axis=0)))
        
        return Frho_wg, self.gd
    
    def _read(self, tar, reads, ws):
        BaseInducedField._read(self, tar, reads, ws)
        
        # Test time
        time = tar['time']
        if abs(time - self.time) >= 1e-9:
            raise IOError('Timestamp is incompatible with calculator.')

        # Allocate
        self.allocate()

        # Read arrays
        if 'n0' in reads:
            big_g = tar.get('n0_G')
            self.gd.distribute(big_g, self.n0_G)
        
        if 'Fn' in reads:
            for w, wread in enumerate(ws):
                big_g = tar.get('Fn_wsG', wread)
                self.gd.distribute(big_g, self.Fn_wsG[w])
        
        if 'eps0' in reads:
            self.eps0_G = self.gd.empty(dtype=float)
            big_g = tar.get('eps0_G')
            self.gd.distribute(big_g, self.eps0_G)
        else:
            self.eps0_G = None


    def _write(self, tar, writes, ws, masters):
        # Swap classical and quantum cells, and shift atom positions for the time of writing
        qmcell = self.atoms.get_cell()
        self.atoms.set_cell(self.fdtd.cl.cell*Bohr) # Set classical cell
        self.atoms.positions = self.atoms.get_positions() + self.fdtd.qm.corner1*Bohr
        BaseInducedField._write(self, tar, writes, ws, masters)
        self.atoms.set_cell(qmcell) # Restore quantum cell to the atoms object
        self.atoms.positions = self.atoms.get_positions() - self.fdtd.qm.corner1*Bohr

        master, domainmaster, bandmaster, kptmaster = masters
        
        # Write time propagation status
        if master:
            tar['time'] = self.time
        
        # Mask, interpolation approach:
        #self.eps0_G = self.fdtd.classical_material.permittivityValue(omega=0.0) - self.fdtd.classical_material.epsInfty
        #self.eps0_G= -interpolator.apply(self.eps0_G)
        
        # Mask, direct approach:
        self.eps0_G = self.fdtd.cl.gd.zeros()
        for component in self.fdtd.classical_material.components:
            self.eps0_G += 1.0 * component.get_mask(gd = self.fdtd.cl.gd, verbose = False)
        
        # Write time propagation arrays
        if 'n0' in writes:
            
            if master:
                tar.add('n0_G',
                        ('ng0', 'ng1', 'ng2'),
                        dtype=float)
            
            if self.fdtd.cl.gd == self.gd:
                big_g = self.gd.collect(self.n0_G)
            else:
                big_g = self.gd.collect(Transformer(self.fdtd.cl.gd, self.gd, self.stencil, float).apply(self.n0_G))
            
            if master:
                tar.fill(big_g)
                
        if 'Fn' in writes:
            if master:
                tar.add('Fn_wsG',
                        ('nw', 'nspins', 'ng0', 'ng1', 'ng2'),
                        dtype=self.dtype)
            for w in ws:
                #big_g = self.gd.collect(self.Fn_wG[w])
                if self.fdtd.cl.gd == self.gd:
                    big_g = self.gd.collect(self.Fn_wsG[w])
                else:
                    big_g = self.gd.collect(Transformer(self.fdtd.cl.gd, self.gd, self.stencil, self.dtype).apply(self.Fn_wsG[w]))
                #big_g = self.gd.collect(interpolator_float.apply(self.n0_G.copy()))
                if master:
                    tar.fill(big_g)
         
        if 'eps0' in writes:
            if master:
                tar.add('eps0_G',
                        ('ng0', 'ng1', 'ng2'),
                        dtype=float)
            #big_g = self.fieldgd.collect(self.eps0_G)
            if self.fdtd.cl.gd == self.gd:
                big_g = self.gd.collect(self.eps0_G)
            else:
                big_g = self.gd.collect(Transformer(self.fdtd.cl.gd, self.gd, self.stencil, float).apply(self.eps0_G))
            if master:
                tar.fill(big_g)
        
        if master and hasattr(self.fdtd, 'qm') and hasattr(self.fdtd.qm, 'corner1'):
            tar.add('corner1_v', ('nv',), self.fdtd.qm.corner1, dtype=float)
            tar.add('corner2_v', ('nv',), self.fdtd.qm.corner2, dtype=float)
        
        self.fdtd.cl.gd.comm.barrier()
            
            


from gpaw.hamiltonian import RealSpaceHamiltonian
from gpaw.solvation.poisson import WeightedFDPoissonSolver
from gpaw.fd_operators import Gradient
import numpy as np


class SolvationRealSpaceHamiltonian(RealSpaceHamiltonian):
    """Realspace Hamiltonian with continuum solvent model.

    See also Section III of
    A. Held and M. Walter, J. Chem. Phys. 141, 174108 (2014).
    """

    def __init__(
        self,
        # solvation related arguments:
        cavity, dielectric, interactions,
        # RealSpaceHamiltonian arguments:
        gd, finegd, nspins, setups, timer, xc, world,
        kptband_comm, grid2grid, vext=None, collinear=True, psolver=None,
        stencil=3
    ):
        """Constructor of SolvationRealSpaceHamiltonian class.

        Additional arguments not present in RealSpaceHamiltonian:
        cavity       -- A Cavity instance.
        dielectric   -- A Dielectric instance.
        interactions -- A list of Interaction instances.
        """
        self.cavity = cavity
        self.dielectric = dielectric
        self.interactions = interactions
        cavity.set_grid_descriptor(finegd)
        dielectric.set_grid_descriptor(finegd)
        for ia in interactions:
            ia.set_grid_descriptor(finegd)
        if psolver is None:
            psolver = WeightedFDPoissonSolver()
        psolver.set_dielectric(self.dielectric)
        self.gradient = None
        RealSpaceHamiltonian.__init__(
            self,
            gd, finegd, nspins, setups, timer, xc, world,
            kptband_comm, vext=vext, collinear=collinear, psolver=psolver,
            stencil=stencil, grid2grid=grid2grid
        )
        for ia in interactions:
            setattr(self, 'E_' + ia.subscript, None)
        self.new_atoms = None
        self.vt_ia_g = None

    def estimate_memory(self, mem):
        RealSpaceHamiltonian.estimate_memory(self, mem)
        solvation = mem.subnode('Solvation')
        for name, obj in [
            ('Cavity', self.cavity),
            ('Dielectric', self.dielectric),
        ] + [('Interaction: ' + ia.subscript, ia) for ia in self.interactions]:
            obj.estimate_memory(solvation.subnode(name))

    def update_atoms(self, atoms):
        self.new_atoms = atoms.copy()

    def initialize(self):
        self.gradient = [
            Gradient(self.finegd, i, 1.0, self.poisson.nn) for i in (0, 1, 2)
        ]
        self.vt_ia_g = self.finegd.zeros()
        self.cavity.allocate()
        self.dielectric.allocate()
        for ia in self.interactions:
            ia.allocate()
        RealSpaceHamiltonian.initialize(self)

    def update(self, density):
        self.timer.start('Hamiltonian')
        if self.vt_sg is None:
            self.timer.start('Initialize Hamiltonian')
            self.initialize()
            self.timer.stop('Initialize Hamiltonian')

        cavity_changed = self.cavity.update(self.new_atoms, density)
        if cavity_changed:
            self.cavity.update_vol_surf()
            self.dielectric.update(self.cavity)

        Epot, Ebar, Eext, Exc = self.update_pseudo_potential(density)
        ia_changed = [
            ia.update(
                self.new_atoms,
                density,
                self.cavity if cavity_changed else None
            ) for ia in self.interactions
        ]
        if np.any(ia_changed):
            self.vt_ia_g.fill(.0)
            for ia in self.interactions:
                if ia.depends_on_el_density:
                    self.vt_ia_g += ia.delta_E_delta_n_g
                if self.cavity.depends_on_el_density:
                    self.vt_ia_g += (ia.delta_E_delta_g_g *
                                     self.cavity.del_g_del_n_g)
        if len(self.interactions) > 0:
            for vt_g in self.vt_sg[:self.nspins]:
                vt_g += self.vt_ia_g
        Eias = [ia.E for ia in self.interactions]

        Ekin1 = self.calculate_kinetic_energy(density)
        W_aL = self.calculate_atomic_hamiltonians(density)
        Ekin2, Epot, Ebar, Eext, Exc = self.update_corrections(
            density, 0.0, Epot, Ebar, Eext, Exc, W_aL
        )

        self.Ekin0 = self.gd.comm.sum(Ekin1) + self.finegd.comm.sum(Ekin2)
        
        energies = np.array([Epot, Ebar, Eext, Exc] + Eias)
        self.timer.start('Communicate energies')
        self.finegd.comm.sum(energies)
        # Make sure that all CPUs have the same energies
        self.world.broadcast(energies, 0)
        self.cavity.communicate_vol_surf(self.world)
        self.timer.stop('Communicate energies')
        (self.Epot, self.Ebar, self.Eext, self.Exc) = energies[:4]
        for E, ia in zip(energies[4:], self.interactions):
            setattr(self, 'E_' + ia.subscript, E)

        # self.Exc += self.Enlxc
        # self.Ekin0 += self.Enlkin

        self.new_atoms = None
        self.timer.stop('Hamiltonian')

    def update_pseudo_potential(self, density):
        ret = RealSpaceHamiltonian.update_pseudo_potential(self, density)
        if not self.cavity.depends_on_el_density:
            return ret
        del_g_del_n_g = self.cavity.del_g_del_n_g
        # XXX optimize numerics
        del_eps_del_g_g = self.dielectric.del_eps_del_g_g
        Veps = -1. / (8. * np.pi) * del_eps_del_g_g * del_g_del_n_g
        Veps *= self.grad_squared(self.vHt_g)
        for vt_g in self.vt_sg[:self.nspins]:
            vt_g += Veps
        return ret

    def calculate_forces(self, dens, F_av):
        # XXX reorganize
        self.el_force_correction(dens, F_av)
        for ia in self.interactions:
            if self.cavity.depends_on_atomic_positions:
                for a, F_v in enumerate(F_av):
                    del_g_del_r_vg = self.cavity.get_del_r_vg(a, dens)
                    for v in (0, 1, 2):
                        F_v[v] -= self.finegd.integrate(
                            ia.delta_E_delta_g_g * del_g_del_r_vg[v],
                            global_integral=False
                        )
            if ia.depends_on_atomic_positions:
                for a, F_v in enumerate(F_av):
                    del_E_del_r_vg = ia.get_del_r_vg(a, dens)
                    for v in (0, 1, 2):
                        F_v[v] -= self.finegd.integrate(
                            del_E_del_r_vg[v],
                            global_integral=False
                        )
        return RealSpaceHamiltonian.calculate_forces(
            self, dens, F_av
        )

    def el_force_correction(self, dens, F_av):
        if not self.cavity.depends_on_atomic_positions:
            return
        del_eps_del_g_g = self.dielectric.del_eps_del_g_g
        fixed = 1. / (8. * np.pi) * del_eps_del_g_g * \
            self.grad_squared(self.vHt_g)  # XXX grad_vHt_g inexact in bmgs
        for a, F_v in enumerate(F_av):
            del_g_del_r_vg = self.cavity.get_del_r_vg(a, dens)
            for v in (0, 1, 2):
                F_v[v] += self.finegd.integrate(
                    fixed * del_g_del_r_vg[v],
                    global_integral=False
                )

    def get_energy(self, occupations):
        self.Ekin = self.Ekin0 + occupations.e_band
        self.S = occupations.e_entropy
        self.Eel = (
            self.Ekin + self.Epot + self.Eext + self.Ebar + self.Exc - self.S
        )
        Etot = self.Eel
        for ia in self.interactions:
            Etot += getattr(self, 'E_' + ia.subscript)
        self.Etot = Etot
        return self.Etot

    def grad_squared(self, x):
        # XXX ugly
        gs = np.empty_like(x)
        tmp = np.empty_like(x)
        self.gradient[0].apply(x, gs)
        np.square(gs, gs)
        self.gradient[1].apply(x, tmp)
        np.square(tmp, tmp)
        gs += tmp
        self.gradient[2].apply(x, tmp)
        np.square(tmp, tmp)
        gs += tmp
        return gs



"""Module defining an eigensolver base-class."""

import numpy as np

from gpaw.utilities.blas import axpy
from gpaw.utilities import unpack
from gpaw.hs_operators import reshape


class Eigensolver:
    def __init__(self, keep_htpsit=True, blocksize=1):
        self.keep_htpsit = keep_htpsit
        self.initialized = False
        self.Htpsit_nG = None
        self.error = np.inf
        self.blocksize = blocksize
        self.orthonormalization_required = True
        
    def initialize(self, wfs):
        self.timer = wfs.timer
        self.world = wfs.world
        self.kpt_comm = wfs.kd.comm
        self.band_comm = wfs.band_comm
        self.dtype = wfs.dtype
        self.bd = wfs.bd
        self.ksl = wfs.diagksl
        self.nbands = wfs.bd.nbands
        self.mynbands = wfs.bd.mynbands
        self.operator = wfs.matrixoperator

        if self.mynbands != self.nbands or self.operator.nblocks != 1:
            self.keep_htpsit = False

        if self.keep_htpsit:
            self.Htpsit_nG = wfs.empty(self.nbands)

        # Preconditioner for the electronic gradients:
        self.preconditioner = wfs.make_preconditioner(self.blocksize)

        for kpt in wfs.kpt_u:
            if kpt.eps_n is None:
                kpt.eps_n = np.empty(self.mynbands)
        
        self.initialized = True

    def reset(self):
        self.initialized = False

    def iterate(self, hamiltonian, wfs):
        """Solves eigenvalue problem iteratively

        This method is inherited by the actual eigensolver which should
        implement *iterate_one_k_point* method for a single iteration of
        a single kpoint.
        """

        if not self.initialized:
            self.initialize(wfs)

        error = 0.0
        for kpt in wfs.kpt_u:
            if not wfs.orthonormalized:
                wfs.overlap.orthonormalize(wfs, kpt)
            e, psit_nG = self.iterate_one_k_point(hamiltonian, wfs, kpt)
            error += e
            if self.orthonormalization_required:
                wfs.overlap.orthonormalize(wfs, kpt, psit_nG)
            else:
                kpt.psit_nG[:] = psit_nG[:]

        wfs.set_orthonormalized(True)

        self.error = self.band_comm.sum(self.kpt_comm.sum(error))

    def iterate_one_k_point(self, hamiltonian, kpt):
        """Implemented in subclasses."""
        raise NotImplementedError

    def calculate_residuals(self, kpt, wfs, hamiltonian, psit_xG, P_axi, eps_x,
                            R_xG, n_x=None, calculate_change=False):
        """Calculate residual.

        From R=Ht*psit calculate R=H*psit-eps*S*psit."""
        
        for R_G, eps, psit_G in zip(R_xG, eps_x, psit_xG):
            axpy(-eps, psit_G, R_G)

        c_axi = {}
        for a, P_xi in P_axi.items():
            dH_ii = unpack(hamiltonian.dH_asp[a][kpt.s])
            dO_ii = hamiltonian.setups[a].dO_ii
            c_xi = (np.dot(P_xi, dH_ii) -
                    np.dot(P_xi * eps_x[:, np.newaxis], dO_ii))
            c_axi[a] = c_xi
        hamiltonian.xc.add_correction(kpt, psit_xG, R_xG, P_axi, c_axi, n_x,
                                      calculate_change)
        wfs.pt.add(R_xG, c_axi, kpt.q)
        
    def subspace_diagonalize(self, hamiltonian, wfs, kpt):
        """Diagonalize the Hamiltonian in the subspace of kpt.psit_nG

        *Htpsit_nG* is a work array of same size as psit_nG which contains
        the local part of the Hamiltonian times psit on exit

        First, the Hamiltonian (defined by *kin*, *vt_sG*, and
        *dH_asp*) is applied to the wave functions, then the *H_nn*
        matrix is calculated and diagonalized, and finally, the wave
        functions (and also Htpsit_nG are rotated.  Also the
        projections *P_ani* are rotated.

        It is assumed that the wave functions *psit_nG* are orthonormal
        and that the integrals of projector functions and wave functions
        *P_ani* are already calculated.

        Return ratated wave functions and H applied to the rotated
        wave functions if self.keep_htpsit is True.
        """

        if self.band_comm.size > 1 and wfs.bd.strided:
            raise NotImplementedError

        self.timer.start('Subspace diag')

        psit_nG = kpt.psit_nG
        P_ani = kpt.P_ani
        dH_asp = hamiltonian.dH_asp

        if self.keep_htpsit:
            Htpsit_nG = reshape(self.Htpsit_nG, psit_nG.shape)
        else:
            Htpsit_nG = None

        def H(psit_xG):
            if self.keep_htpsit:
                result_xG = Htpsit_nG
            else:
                result_xG = reshape(self.operator.work1_xG, psit_xG.shape)
            wfs.apply_pseudo_hamiltonian(kpt, hamiltonian, psit_xG,
                                         result_xG)
            return result_xG

        def dH(a, P_ni):
            return np.dot(P_ni, unpack(dH_asp[a][kpt.s]))

        self.timer.start('calc_h_matrix')
        H_nn = self.operator.calculate_matrix_elements(psit_nG, P_ani,
                                                       H, dH)
        hamiltonian.xc.correct_hamiltonian_matrix(kpt, H_nn)
        self.timer.stop('calc_h_matrix')

        diagonalization_string = repr(self.ksl)
        wfs.timer.start(diagonalization_string)
        self.ksl.diagonalize(H_nn, kpt.eps_n)
        # H_nn now contains the result of the diagonalization.
        wfs.timer.stop(diagonalization_string)

        self.timer.start('rotate_psi')
        psit_nG = self.operator.matrix_multiply(H_nn, psit_nG, P_ani)
        if self.keep_htpsit:
            Htpsit_nG = self.operator.matrix_multiply(H_nn, Htpsit_nG,
                                                      out_nG=kpt.psit_nG)

        # Rotate orbital dependent XC stuff:
        hamiltonian.xc.rotate(kpt, H_nn)

        self.timer.stop('rotate_psi')
        self.timer.stop('Subspace diag')

        return psit_nG, Htpsit_nG

    def estimate_memory(self, mem, wfs):
        gridmem = wfs.bytes_per_wave_function()

        keep_htpsit = self.keep_htpsit and (wfs.bd.mynbands == wfs.bd.nbands)

        if keep_htpsit:
            mem.subnode('Htpsit', wfs.bd.nbands * gridmem)
        else:
            mem.subnode('No Htpsit', 0)

        mem.subnode('eps_n', wfs.bd.mynbands * mem.floatsize)
        mem.subnode('eps_N', wfs.bd.nbands * mem.floatsize)
        mem.subnode('Preconditioner', 4 * gridmem)
        mem.subnode('Work', gridmem)



import numpy as np
import os
from gpaw.io.hdf5_highlevel import File, HyperslabSelection
from gpaw.mpi import world, rank, size

# data to be written
attrs = {'int' : 42, 'float' : 6.2, 'complex' : 1.0 + 2.0j, 
         'bool1' : False, 'bool2' : True, 'data' : np.array(((1,2),(3,4)))}
datasize = 4
data = {}
for dtype in ['int', 'float', 'complex']:
    data[dtype] = np.arange(rank * datasize, rank * datasize + datasize, 
                            dtype = dtype)
data['bool'] = np.array([True, False, False, True])

def write(attrs, data, file, parallel):
    file_title = 'test file'
    file.attrs['title'] = file_title
    grp = file.create_group('group1')
    # write attributes
    for key, val in attrs.items():
        grp.attrs[key] = val
    # write datasets
    shape = (datasize * size,)
    for key, val in data.items():
        dset = grp.create_dataset(key, shape, val.dtype) 
        indices = slice(rank * datasize, (rank + 1) * datasize)
        selection = HyperslabSelection(indices, dset.shape)
        dset.write(val, selection, collective=parallel)

def read(attrs, data, file, parallel):
    file_title = file.attrs['title']
    assert(file_title == 'test file')
    grp = file['group1']
    # read attributes
    for key, val in attrs.items():
        new_val = grp.attrs[key]
        if key == 'data':
            assert((new_val == val).all())
        else:
            assert(new_val == val)
    # read data
    for key, val in data.items():
        dset = grp[key]
        indices = slice((size - rank - 1) * datasize, 
                        (size - rank) * datasize)
        selection = HyperslabSelection(indices, dset.shape)
        new_val = np.ndarray(selection.mshape, dset.dtype, order='C')
        dset.read(new_val, selection, collective=parallel)
        assert((new_val == val).all)
    # read data in reversed order
    for key in data:
        if key == 'bool':
            continue
        dset = grp[key]
        indices = slice((size - rank - 1) * datasize, 
                        (size - rank) * datasize)
        selection = HyperslabSelection(indices, dset.shape)
        new_val = np.ndarray(selection.mshape, dset.dtype, order='C')
        dset.read(new_val, selection, collective=parallel)
        ref_val = np.arange((size - rank - 1) * datasize, 
                            (size - rank) * datasize, dtype=key)
        assert((new_val == ref_val).all)

if world.size > 1:
    comm = world.get_c_object()
    parallel = True
else:
    comm = None
    parallel = False

file = File('tmp.hdf5', 'w', comm=comm)
write(attrs, data, file, parallel)
file.close()

file = File('tmp.hdf5', 'r', comm=comm)
read(attrs, data, file, parallel)
file.close()

if rank == 0:
    os.remove('tmp.hdf5')



"""Test automatically write out of restart files"""

import os
from glob import glob
from gpaw import GPAW
from ase import Atom, Atoms
from gpaw.test import equal

restart = 'gpaw-restart'
result  = 'gpaw-result'

# H atom:
H = Atoms([Atom('H')])
H.center(vacuum=3.0)

calc = GPAW(gpts=(32, 32, 32), nbands=1)
calc.attach(calc.write, 4, restart)
H.set_calculator(calc)
e = H.get_potential_energy()
niter = calc.get_number_of_iterations()
calc.write(result)



# the two files should be equal
from gpaw.mpi import rank
if rank == 0:
    for f in ['gpaw-restart', 'gpaw-result']:
        os.system('rm -rf %s; mkdir %s; cd %s; tar xf ../%s.gpw' %
                  (f, f, f, f))

    # make a list of the files to compare
    files_restart = glob(restart+'/*')
    files_result = glob(result+'/*')

    for f1, f2 in zip(files_restart, files_result):
        # cmp is a byte-by-byte comparison, more portable than diff
        assert os.system('cmp %s %s' % (f1, f2)) == 0
    os.system('rm -rf gpaw-restart gpaw-result')

energy_tolerance = 0.00006
niter_tolerance = 0
equal(e, 0.0451789, energy_tolerance)



from __future__ import print_function
from math import sqrt
import numpy as np
import gpaw.mpi as mpi
MASTER = mpi.MASTER

from ase.units import Hartree
from ase.utils.timing import Timer

import gpaw.mpi as mpi
#from gpaw.poisson import PoissonSolver
from gpaw.output import get_txt
from gpaw.lrtddft.kssingle import KSSingles
from gpaw.transformers import Transformer
from gpaw.utilities import pack
from gpaw.utilities.lapack import diagonalize
from gpaw.xc import XC

"""This module defines a Omega Matrix class."""


class OmegaMatrix:

    """
    Omega matrix in Casidas linear response formalism

    Parameters
      - calculator: the calculator object the ground state calculation
      - kss: the Kohn-Sham singles object
      - xc: the exchange correlation approx. to use
      - derivativeLevel: which level i of d^i Exc/dn^i to use
      - numscale: numeric epsilon for derivativeLevel=0,1
      - filehandle: the oject can be read from a filehandle
      - txt: output stream or file name
      - finegrid: level of fine grid to use. 0: nothing, 1 for poisson only,
        2 everything on the fine grid
    """

    def __init__(self,
                 calculator=None,
                 kss=None,
                 xc=None,
                 derivativeLevel=None,
                 numscale=0.001,
                 filehandle=None,
                 txt=None,
                 finegrid=2,
                 eh_comm=None,
                 ):

        if not txt and calculator:
            txt = calculator.txt
        self.txt = get_txt(txt, mpi.rank)

        if eh_comm is None:
            eh_comm = mpi.serial_comm

        self.eh_comm = eh_comm

        if filehandle is not None:
            self.kss = kss
            self.read(fh=filehandle)
            return None

        self.fullkss = kss
        self.finegrid = finegrid

        if calculator is None:
            return

        self.paw = calculator
        wfs = self.paw.wfs

        # handle different grid possibilities
        self.restrict = None
        # self.poisson = PoissonSolver(nn=self.paw.hamiltonian.poisson.nn)
        self.poisson = calculator.hamiltonian.poisson
        if finegrid:
            self.poisson.set_grid_descriptor(self.paw.density.finegd)
            self.poisson.initialize()

            self.gd = self.paw.density.finegd
            if finegrid == 1:
                self.gd = wfs.gd
        else:
            self.poisson.set_grid_descriptor(wfs.gd)
            self.poisson.initialize()
            self.gd = wfs.gd
        self.restrict = Transformer(self.paw.density.finegd, wfs.gd,
                                    self.paw.input_parameters.stencils[1]
                                    ).apply

        if xc == 'RPA':
            xc = None  # enable RPA as keyword
        if xc is not None:
            self.xc = XC(xc)
            self.xc.initialize(self.paw.density, self.paw.hamiltonian,
                               wfs, self.paw.occupations)

            # check derivativeLevel
            if derivativeLevel is None:
                derivativeLevel = \
                    self.xc.get_functional().get_max_derivative_level()
            self.derivativeLevel = derivativeLevel
        else:
            self.xc = None

        self.numscale = numscale

        self.singletsinglet = False
        if kss.nvspins < 2 and kss.npspins < 2:
            # this will be a singlet to singlet calculation only
            self.singletsinglet = True

        nij = len(kss)
        self.Om = np.zeros((nij, nij))
        self.get_full()

    def get_full(self):

        self.paw.timer.start('Omega RPA')
        self.get_rpa()
        self.paw.timer.stop()

        if self.xc is not None:
            self.paw.timer.start('Omega XC')
            self.get_xc()
            self.paw.timer.stop()

        self.eh_comm.sum(self.Om)
        self.full = self.Om

    def get_xc(self):
        """Add xc part of the coupling matrix"""

        # shorthands
        paw = self.paw
        wfs = paw.wfs
        gd = paw.density.finegd
        eh_comm = self.eh_comm

        fg = self.finegrid is 2
        kss = self.fullkss
        nij = len(kss)

        Om_xc = self.Om
        # initialize densities
        # nt_sg is the smooth density on the fine grid with spin index

        if kss.nvspins == 2:
            # spin polarised ground state calc.
            nt_sg = paw.density.nt_sg
        else:
            # spin unpolarised ground state calc.
            if kss.npspins == 2:
                # construct spin polarised densities
                nt_sg = np.array([.5 * paw.density.nt_sg[0],
                                  .5 * paw.density.nt_sg[0]])
            else:
                nt_sg = paw.density.nt_sg
        # check if D_sp have been changed before
        D_asp = {}
        for a, D_sp in self.paw.density.D_asp.items():
            if len(D_sp) != kss.npspins:
                if len(D_sp) == 1:
                    D_asp[a] = np.array([0.5 * D_sp[0], 0.5 * D_sp[0]])
                else:
                    D_asp[a] = np.array([D_sp[0] + D_sp[1]])
            else:
                D_asp[a] = D_sp.copy()

        # restrict the density if needed
        if fg:
            nt_s = nt_sg
        else:
            nt_s = self.gd.zeros(nt_sg.shape[0])
            for s in range(nt_sg.shape[0]):
                self.restrict(nt_sg[s], nt_s[s])
            gd = paw.density.gd

        # initialize vxc or fxc

        if self.derivativeLevel == 0:
            raise NotImplementedError
            if kss.npspins == 2:
                v_g = nt_sg[0].copy()
            else:
                v_g = nt_sg.copy()
        elif self.derivativeLevel == 1:
            pass
        elif self.derivativeLevel == 2:
            fxc_sg = np.zeros(nt_sg.shape)
            self.xc.calculate_fxc(gd, nt_sg, fxc_sg)
        else:
            raise ValueError('derivativeLevel can only be 0,1,2')

# self.paw.my_nuclei = []

        ns = self.numscale
        xc = self.xc
        print('XC', nij, 'transitions', file=self.txt)
        for ij in range(eh_comm.rank, nij, eh_comm.size):
            print('XC kss[' + '%d' % ij + ']', file=self.txt)

            timer = Timer()
            timer.start('init')
            timer2 = Timer()

            if self.derivativeLevel >= 1:
                # vxc is available
                # We use the numerical two point formula for calculating
                # the integral over fxc*n_ij. The results are
                # vvt_s        smooth integral
                # nucleus.I_sp atom based correction matrices (pack2)
                #              stored on each nucleus
                timer2.start('init v grids')
                vp_s = np.zeros(nt_s.shape, nt_s.dtype.char)
                vm_s = np.zeros(nt_s.shape, nt_s.dtype.char)
                if kss.npspins == 2:  # spin polarised
                    nv_s = nt_s.copy()
                    nv_s[kss[ij].pspin] += ns * kss[ij].get(fg)
                    xc.calculate(gd, nv_s, vp_s)
                    nv_s = nt_s.copy()
                    nv_s[kss[ij].pspin] -= ns * kss[ij].get(fg)
                    xc.calculate(gd, nv_s, vm_s)
                else:  # spin unpolarised
                    nv = nt_s + ns * kss[ij].get(fg)
                    xc.calculate(gd, nv, vp_s)
                    nv = nt_s - ns * kss[ij].get(fg)
                    xc.calculate(gd, nv, vm_s)
                vvt_s = (0.5 / ns) * (vp_s - vm_s)
                timer2.stop()

                # initialize the correction matrices
                timer2.start('init v corrections')
                I_asp = {}
                for a, P_ni in wfs.kpt_u[kss[ij].spin].P_ani.items():
                    # create the modified density matrix
                    Pi_i = P_ni[kss[ij].i]
                    Pj_i = P_ni[kss[ij].j]
                    P_ii = np.outer(Pi_i, Pj_i)
                    # we need the symmetric form, hence we can pack
                    P_p = pack(P_ii)
                    D_sp = D_asp[a].copy()
                    D_sp[kss[ij].pspin] -= ns * P_p
                    setup = wfs.setups[a]
                    I_sp = np.zeros_like(D_sp)
                    self.xc.calculate_paw_correction(setup, D_sp, I_sp)
                    I_sp *= -1.0
                    D_sp = D_asp[a].copy()
                    D_sp[kss[ij].pspin] += ns * P_p
                    self.xc.calculate_paw_correction(setup, D_sp, I_sp)
                    I_sp /= 2.0 * ns
                    I_asp[a] = I_sp
                timer2.stop()

            timer.stop()
            t0 = timer.get_time('init')
            timer.start(ij)

            for kq in range(ij, nij):
                weight = self.weight_Kijkq(ij, kq)

                if self.derivativeLevel == 0:
                    # only Exc is available

                    if kss.npspins == 2:  # spin polarised
                        nv_g = nt_sg.copy()
                        nv_g[kss[ij].pspin] += kss[ij].get(fg)
                        nv_g[kss[kq].pspin] += kss[kq].get(fg)
                        Excpp = xc.get_energy_and_potential(
                            nv_g[0], v_g, nv_g[1], v_g)
                        nv_g = nt_sg.copy()
                        nv_g[kss[ij].pspin] += kss[ij].get(fg)
                        nv_g[kss[kq].pspin] -= kss[kq].get(fg)
                        Excpm = xc.get_energy_and_potential(
                            nv_g[0], v_g, nv_g[1], v_g)
                        nv_g = nt_sg.copy()
                        nv_g[kss[ij].pspin] -=\
                            kss[ij].get(fg)
                        nv_g[kss[kq].pspin] +=\
                            kss[kq].get(fg)
                        Excmp = xc.get_energy_and_potential(
                            nv_g[0], v_g, nv_g[1], v_g)
                        nv_g = nt_sg.copy()
                        nv_g[kss[ij].pspin] -= \
                            kss[ij].get(fg)
                        nv_g[kss[kq].pspin] -=\
                            kss[kq].get(fg)
                        Excpp = xc.get_energy_and_potential(
                            nv_g[0], v_g, nv_g[1], v_g)
                    else:  # spin unpolarised
                        nv_g = nt_sg + ns * kss[ij].get(fg)\
                            + ns * kss[kq].get(fg)
                        Excpp = xc.get_energy_and_potential(nv_g, v_g)
                        nv_g = nt_sg + ns * kss[ij].get(fg)\
                            - ns * kss[kq].get(fg)
                        Excpm = xc.get_energy_and_potential(nv_g, v_g)
                        nv_g = nt_sg - ns * kss[ij].get(fg)\
                            + ns * kss[kq].get(fg)
                        Excmp = xc.get_energy_and_potential(nv_g, v_g)
                        nv_g = nt_sg - ns * kss[ij].get(fg)\
                            - ns * kss[kq].get(fg)
                        Excmm = xc.get_energy_and_potential(nv_g, v_g)

                    Om_xc[ij, kq] += weight *\
                        0.25 * \
                        (Excpp - Excmp - Excpm + Excmm) / (ns * ns)

                elif self.derivativeLevel == 1:
                    # vxc is available

                    timer2.start('integrate')
                    Om_xc[ij, kq] += weight *\
                        self.gd.integrate(kss[kq].get(fg) *
                                          vvt_s[kss[kq].pspin])
                    timer2.stop()

                    timer2.start('integrate corrections')
                    Exc = 0.
                    for a, P_ni in wfs.kpt_u[kss[kq].spin].P_ani.items():
                        # create the modified density matrix
                        Pk_i = P_ni[kss[kq].i]
                        Pq_i = P_ni[kss[kq].j]
                        P_ii = np.outer(Pk_i, Pq_i)
                        # we need the symmetric form, hence we can pack
                        # use pack as I_sp used pack2
                        P_p = pack(P_ii)
                        Exc += np.dot(I_asp[a][kss[kq].pspin], P_p)
                    Om_xc[ij, kq] += weight * self.gd.comm.sum(Exc)
                    timer2.stop()

                elif self.derivativeLevel == 2:
                    # fxc is available
                    if kss.npspins == 2:  # spin polarised
                        Om_xc[ij, kq] += weight *\
                            gd.integrate(kss[ij].get(fg) *
                                         kss[kq].get(fg) *
                                         fxc_sg[kss[ij].pspin, kss[kq].pspin])
                    else:  # spin unpolarised
                        Om_xc[ij, kq] += weight *\
                            gd.integrate(kss[ij].get(fg) *
                                         kss[kq].get(fg) *
                                         fxc_sg)

                    # XXX still numeric derivatives for local terms
                    timer2.start('integrate corrections')
                    Exc = 0.
                    for a, P_ni in wfs.kpt_u[kss[kq].spin].P_ani.items():
                        # create the modified density matrix
                        Pk_i = P_ni[kss[kq].i]
                        Pq_i = P_ni[kss[kq].j]
                        P_ii = np.outer(Pk_i, Pq_i)
                        # we need the symmetric form, hence we can pack
                        # use pack as I_sp used pack2
                        P_p = pack(P_ii)
                        Exc += np.dot(I_asp[a][kss[kq].pspin], P_p)
                    Om_xc[ij, kq] += weight * self.gd.comm.sum(Exc)
                    timer2.stop()

                if ij != kq:
                    Om_xc[kq, ij] = Om_xc[ij, kq]

            timer.stop()
# timer2.write()
            if ij < (nij - 1):
                print('XC estimated time left',
                      self.time_left(timer, t0, ij, nij), file=self.txt)

    def Coulomb_integral_kss(self, kss_ij, kss_kq, phit, rhot,
                             timer=None):
        # smooth part
        if timer:
            timer.start('integrate')
        I = self.gd.integrate(rhot * phit)
        if timer:
            timer.stop()
            timer.start('integrate corrections 2')

        wfs = self.paw.wfs
        Pij_ani = wfs.kpt_u[kss_ij.spin].P_ani
        Pkq_ani = wfs.kpt_u[kss_kq.spin].P_ani

        # Add atomic corrections
        Ia = 0.0
        for a, Pij_ni in Pij_ani.items():
            Pi_i = Pij_ni[kss_ij.i]
            Pj_i = Pij_ni[kss_ij.j]
            Dij_ii = np.outer(Pi_i, Pj_i)
            Dij_p = pack(Dij_ii)
            Pk_i = Pkq_ani[a][kss_kq.i]
            Pq_i = Pkq_ani[a][kss_kq.j]
            Dkq_ii = np.outer(Pk_i, Pq_i)
            Dkq_p = pack(Dkq_ii)
            C_pp = wfs.setups[a].M_pp
            #   ----
            # 2 >      P   P  C    P  P
            #   ----    ip  jr prst ks qt
            #   prst
            Ia += 2.0 * np.dot(Dkq_p, np.dot(C_pp, Dij_p))
        I += self.gd.comm.sum(Ia)
        if timer:
            timer.stop()

        return I

    def get_rpa(self):
        """calculate RPA part of the omega matrix"""

        # shorthands
        kss = self.fullkss
        finegrid = self.finegrid
        eh_comm = self.eh_comm

        # calculate omega matrix
        nij = len(kss)
        print('RPA', nij, 'transitions', file=self.txt)

        Om = self.Om

        for ij in range(eh_comm.rank, nij, eh_comm.size):
            print('RPA kss[' + '%d' % ij + ']=', kss[ij], file=self.txt)

            timer = Timer()
            timer.start('init')
            timer2 = Timer()

            # smooth density including compensation charges
            timer2.start('with_compensation_charges 0')
            rhot_p = kss[ij].with_compensation_charges(
                finegrid is not 0)
            timer2.stop()

            # integrate with 1/|r_1-r_2|
            timer2.start('poisson')
            phit_p = np.zeros(rhot_p.shape, rhot_p.dtype.char)
            self.poisson.solve(phit_p, rhot_p, charge=None)
            timer2.stop()

            timer.stop()
            t0 = timer.get_time('init')
            timer.start(ij)

            if finegrid == 1:
                rhot = kss[ij].with_compensation_charges()
                phit = self.gd.zeros()
# print "shapes 0=",phit.shape,rhot.shape
                self.restrict(phit_p, phit)
            else:
                phit = phit_p
                rhot = rhot_p

            for kq in range(ij, nij):
                if kq != ij:
                    # smooth density including compensation charges
                    timer2.start('kq with_compensation_charges')
                    rhot = kss[kq].with_compensation_charges(
                        finegrid is 2)
                    timer2.stop()

                pre = 2 * sqrt(kss[ij].get_energy() * kss[kq].get_energy() *
                               kss[ij].get_weight() * kss[kq].get_weight())
                I = self.Coulomb_integral_kss(kss[ij], kss[kq],
                                              rhot, phit, timer2)
                Om[ij, kq] = pre * I

                if ij == kq:
                    Om[ij, kq] += kss[ij].get_energy() ** 2
                else:
                    Om[kq, ij] = Om[ij, kq]

            timer.stop()
# timer2.write()
            if ij < (nij - 1):
                t = timer.get_time(ij)  # time for nij-ij calculations
                t = .5 * t * \
                    (nij - ij)  # estimated time for n*(n+1)/2, n=nij-(ij+1)
                print('RPA estimated time left',
                      self.timestring(t0 * (nij - ij - 1) + t), file=self.txt)

    def singlets_triplets(self):
        """Split yourself into singlet and triplet transitions"""

        assert(self.fullkss.npspins == 2)
        assert(self.fullkss.nvspins == 1)

        # strip kss from down spins
        skss = KSSingles()
        tkss = KSSingles()
        map = []
        for ij, ks in enumerate(self.fullkss):
            if ks.pspin == ks.spin:
                skss.append((ks + ks) / sqrt(2))
                tkss.append((ks - ks) / sqrt(2))
                map.append(ij)

        nkss = len(skss)

        # define the singlet and the triplet omega-matrixes
        sOm = OmegaMatrix(kss=skss)
        sOm.full = np.empty((nkss, nkss))
        tOm = OmegaMatrix(kss=tkss)
        tOm.full = np.empty((nkss, nkss))
        for ij in range(nkss):
            for kl in range(nkss):
                sOm.full[ij, kl] = (self.full[map[ij], map[kl]] +
                                    self.full[map[ij], nkss + map[kl]])
                tOm.full[ij, kl] = (self.full[map[ij], map[kl]] -
                                    self.full[map[ij], nkss + map[kl]])
        return sOm, tOm

    def timestring(self, t):
        ti = int(t + 0.5)
        td = ti // 86400
        st = ''
        if td > 0:
            st += '%d' % td + 'd'
            ti -= td * 86400
        th = ti // 3600
        if th > 0:
            st += '%d' % th + 'h'
            ti -= th * 3600
        tm = ti // 60
        if tm > 0:
            st += '%d' % tm + 'm'
            ti -= tm * 60
        st += '%d' % ti + 's'
        return st

    def time_left(self, timer, t0, ij, nij):
        t = timer.get_time(ij)  # time for nij-ij calculations
        t = .5 * t * (nij - ij)  # estimated time for n*(n+1)/2, n=nij-(ij+1)
        return self.timestring(t0 * (nij - ij - 1) + t)

    def get_map(self, istart=None, jend=None, energy_range=None):
        """Return the reduction map for the given requirements"""

        self.istart = istart
        self.jend = jend
        if istart is None and jend is None and energy_range is None:
            return None, self.fullkss

        # reduce the matrix
        print('# diagonalize: %d transitions original'
              % len(self.fullkss), file=self.txt)

        if energy_range is None:
            if istart is None:
                istart = self.kss.istart
            if self.fullkss.istart > istart:
                raise RuntimeError('istart=%d has to be >= %d' %
                                   (istart, self.kss.istart))
            if jend is None:
                jend = self.kss.jend
            if self.fullkss.jend < jend:
                raise RuntimeError('jend=%d has to be <= %d' %
                                   (jend, self.kss.jend))

        else:
            try:
                emin, emax = energy_range
            except:
                emax = energy_range
                emin = 0.
            emin /= Hartree
            emax /= Hartree

        map = []
        kss = KSSingles()
        for ij, k in zip(range(len(self.fullkss)), self.fullkss):
            if energy_range is None:
                if k.i >= istart and k.j <= jend:
                    kss.append(k)
                    map.append(ij)
            else:
                if k.energy >= emin and k.energy < emax:
                    kss.append(k)
                    map.append(ij)
        kss.update()
        print('# diagonalize: %d transitions now' % len(kss), file=self.txt)

        return map, kss

    def diagonalize(self, istart=None, jend=None, energy_range=None,
                    TDA=False):
        """Evaluate Eigenvectors and Eigenvalues:"""

        if TDA:
            raise NotImplementedError

        map, kss = self.get_map(istart, jend, energy_range)
        nij = len(kss)
        if map is None:
            evec = self.full.copy()
        else:
            evec = np.zeros((nij, nij))
            for ij in range(nij):
                for kq in range(nij):
                    evec[ij, kq] = self.full[map[ij], map[kq]]
        assert(len(evec) > 0)

        self.eigenvectors = evec
        self.eigenvalues = np.zeros((len(kss)))
        self.kss = kss
        diagonalize(self.eigenvectors, self.eigenvalues)

    def Kss(self, kss=None):
        """Set and get own Kohn-Sham singles"""
        if kss is not None:
            self.fullkss = kss
        if(hasattr(self, 'fullkss')):
            return self.fullkss
        else:
            return None

    def read(self, filename=None, fh=None):
        """Read myself from a file"""
        if fh is None:
            f = open(filename, 'r')
        else:
            f = fh

        f.readline()
        nij = int(f.readline())
        full = np.zeros((nij, nij))
        for ij in range(nij):
            l = f.readline().split()
            for kq in range(ij, nij):
                full[ij, kq] = float(l[kq - ij])
                full[kq, ij] = full[ij, kq]
        self.full = full

        if fh is None:
            f.close()

    def write(self, filename=None, fh=None):
        """Write current state to a file."""
        if mpi.rank == mpi.MASTER:
            if fh is None:
                f = open(filename, 'w')
            else:
                f = fh

            f.write('# OmegaMatrix\n')
            nij = len(self.fullkss)
            f.write('%d\n' % nij)
            for ij in range(nij):
                for kq in range(ij, nij):
                    f.write(' %g' % self.full[ij, kq])
                f.write('\n')

            if fh is None:
                f.close()

    def weight_Kijkq(self, ij, kq):
        """weight for the coupling matrix terms"""
        kss = self.fullkss
        return 2. * sqrt(kss[ij].get_energy() * kss[kq].get_energy() *
                         kss[ij].get_weight() * kss[kq].get_weight())

    def __str__(self):
        str = '<OmegaMatrix> '
        if hasattr(self, 'eigenvalues'):
            str += 'dimension ' + ('%d' % len(self.eigenvalues))
            str += '\neigenvalues: '
            for ev in self.eigenvalues:
                str += ' ' + ('%f' % (sqrt(ev) * Hartree))
        return str



def agts(queue):
    run = queue.add('pbe_gpaw_pw.py Al', ncpus=4,
                    queueopts='-l nodes=1:ppn=4:opteron4', walltime=60)
    if 0:  # run when new setups ready
        analyse = queue.add('analyse.py dcdft_pbe_gpaw_pw',
                            ncpus=1, walltime=10, deps=run,
                            creates=['dcdft_pbe_gpaw_pw.csv',
                                     'dcdft_pbe_gpaw_pw.txt',
                                     'dcdft_pbe_gpaw_pw_Delta.txt',
                                     'dcdft_pbe_gpaw_pw_raw.csv'])
        verify = queue.add('pbe_gpaw_pw_verify.py',
                           ncpus=1, walltime=10, deps=[analyse])



from ase.lattice import bulk
from gpaw import GPAW
si = bulk('Si', 'diamond', a=5.5, cubic=not True)
si.set_calculator(GPAW(setups='ah', kpts=(2, 2, 2)))
si.get_potential_energy()
si.calc.write('Si.gpw', 'all')
GPAW('Si.gpw')
